package edu.ucla.cs.dbevolution;

import java.io.File;
import java.io.PrintStream;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import org.netbeans.lib.cvsclient.CVSRoot;
import org.netbeans.lib.cvsclient.Client;
import org.netbeans.lib.cvsclient.admin.StandardAdminHandler;
import org.netbeans.lib.cvsclient.command.GlobalOptions;
import org.netbeans.lib.cvsclient.command.checkout.CheckoutCommand;
import org.netbeans.lib.cvsclient.command.log.LogInformation;
import org.netbeans.lib.cvsclient.command.log.RlogCommand;
import org.netbeans.lib.cvsclient.connection.AuthenticationException;
import org.netbeans.lib.cvsclient.connection.PServerConnection;
import org.netbeans.lib.cvsclient.event.CVSAdapter;
import org.netbeans.lib.cvsclient.event.FileAddedEvent;
import org.netbeans.lib.cvsclient.event.FileInfoEvent;
import org.netbeans.lib.cvsclient.event.MessageEvent;

public class CvsOperations implements Repository{
	
	protected Vector<DbVersion> revisions;
	
	//implements the GetRevision method for the cvs repository
	public Vector<DbVersion> GetRevisions(){
		
	
	    GlobalOptions globalOptions = new GlobalOptions();
	    globalOptions.setCVSRoot(CVSRoot.parse(ApplicationParameters.getCvsRoot()).toString());
	    CVSRoot rt = CVSRoot.parse(globalOptions.getCVSRoot());
	    
		try{
			//create the connection and verify it
			PServerConnection c = new PServerConnection(rt);
		    c.setUserName(rt.getUserName());
		    c.setEncodedPassword(rt.getPassword());
		    c.setHostName(rt.getHostName());
		    c.setRepository(rt.getRepository());    
		    c.verify();
		    c.open();
		    
		    //cvs client creation
		    Client client = new Client(c, new StandardAdminHandler());     
		    client.setLocalPath(ApplicationParameters.getCvsLocalPath());
		    client.getEventManager().addCVSListener(new LogListener());
		
		    RlogCommand rlog = new RlogCommand();
		    rlog.setModule(ApplicationParameters.getCvsmodule());
		    rlog.setNoTags(true);
		    //set the default (main) branch, equals to -b into the command line
		    rlog.setDefaultBranch(true);
		    client.executeCommand(rlog,globalOptions);
		    c.close();
		}
		catch(Exception e){
			e.printStackTrace();
		}
		//Iterator<DbVersion> itRevision = revisions.iterator(); 
		//while(itRevision.hasNext()){
		//	itRevision.next();

		return revisions;
	}
	
	
	/**
	 * implements the DownloadRevision method for the cvs repository
	 */
	public void DownloadRevision(String revision){
		
		File f;
		String sqlExt = ".sql";
		
		//set a system property so as to keep the cvs root 
		//System.setProperty("cvs.root",":pserver:anonymous@zabbix.cvs.sourceforge.net:/cvsroot/zabbix");
		
	    GlobalOptions globalOptions = new GlobalOptions();
	    globalOptions.setCVSRoot(CVSRoot.parse(ApplicationParameters.getCvsRoot()).toString());

	    CVSRoot rt = CVSRoot.parse(globalOptions.getCVSRoot());
    
	    
		try{
			//create the connection and verify it
			PServerConnection c = new PServerConnection(rt);
		    c.setUserName(rt.getUserName());
		    c.setEncodedPassword(rt.getPassword());
		    c.setHostName(rt.getHostName());
		    c.setRepository(rt.getRepository());    
		    c.verify();
		    c.open();
		    
		    //cvs client creation
		    Client client = new Client(c, new StandardAdminHandler());     
		    client.setLocalPath(ApplicationParameters.getCvsLocalPath());
		    client.getEventManager().addCVSListener(new ExportListener());
	
		    //checkout command 
		    CheckoutCommand check = new CheckoutCommand();
		    //set the check out command to write to the standard output
		    check.setPipeToOutput(true);
		    check.setCheckoutByRevision(revision);
		    check.setModule(ApplicationParameters.getCvsmodule());
		    
		    //save the old standard output
		    PrintStream oldstdOut = System.out;
		    
		    //the cvs checkout command has the possibility to print (save) the file checked out to the standard output
		    //in this way we redirect the standard output to the file we want to save and we have exported the desired schema
		    f = new File(ApplicationParameters.getPathtoschema() + revision + sqlExt);
		    //FileOutputStream fo = new FileOutputStream(f);
		    System.setOut(new PrintStream(f));
		    
		    client.executeCommand(check,globalOptions);
		    c.close();
		    
		    //set back the old std output
		    System.setOut(oldstdOut);
		}
		catch(AuthenticationException aE ){
			aE.printStackTrace();
		}
		catch(NullPointerException nE){
			nE.printStackTrace();
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
	}
	
	private class ExportListener extends CVSAdapter
	{
	    /**
	     * Stores a tagged line
	     */
	    private final StringBuffer taggedLine = new StringBuffer();

	    /**
	     * Called when the server wants to send a message to be displayed to
	     * the user. The message is only for information purposes and clients
	     * can choose to ignore these messages if they wish.
	     * @param e the event
	     */
	    public void messageSent(MessageEvent e)
	    {
	        String line = e.getMessage();
	        PrintStream stream = e.isError() ? System.err
	                                         : System.out;

	        if (e.isTagged())
	        {
	            String message = "";//e.parseTaggedMessage(taggedLine);
		    // if we get back a non-null line, we have something
		    // to output. Otherwise, there is more to come and we
		    // should do nothing yet.
	            if (message != null)
	            {
	                stream.println(message);
	            }
	        }
	        else
	        {
	            stream.println(line);
	        }
	    }
	    
	    public void fileAdded(FileAddedEvent e) {
			System.out.println(e.getFilePath());
		}
	}
	
	private class LogListener extends CVSAdapter{
	    
	    public void fileInfoGenerated(FileInfoEvent e){
	    	
	    	revisions = new Vector<DbVersion>();
	    	
	    	LogInformation linfo = (LogInformation)e.getInfoContainer();
	    	Iterator itRevisions = linfo.getRevisionList().iterator();
	    	while(itRevisions.hasNext()){
	    		LogInformation.Revision rev = (LogInformation.Revision)itRevisions.next(); 
	    		DbVersion version = new DbVersion();
	    		version.set_dbName(ApplicationParameters.getDbbasename());
	    		version.set_dbVersion(rev.getNumber());
	    		//version.set_dbVersionName(cleanRevisionNumber(rev.getNumber()));
	    		version.set_dateFrom(rev.getDate());
	    		revisions.add(version);
	    		System.out.println("Revision added");
	    	}
			/*for(int i=0;i<revisions.size()-1;i++){	//the last element will have the date_to field set to NULL 
				((DbVersion)revisions.get(i)).set_dateTo(((DbVersion)revisions.get(i+1)).get_dateFrom());
				//System.out.println("SET DATE TO");
			}*/
	    	for(int i=revisions.size()-1;i>0;i--){	//the last element will have the date_to field set to NULL 
				((DbVersion)revisions.get(i)).set_dateTo(((DbVersion)revisions.get(i-1)).get_dateFrom());
				//System.out.println("SET DATE TO");
			}
			//set the Date_to field of the last element to TODAY otherwise, the method that insert the values in the db throws an exception
			//((DbVersion)revisions.lastElement()).set_dateTo(new Date());
	    	((DbVersion)revisions.firstElement()).set_dateTo(new Date());
	    }
	}
		 
}
