package edu.ucla.cs.dbevolution;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.statistics.MonthRevision;
import edu.ucla.cs.statistics.Statistic;
import edu.ucla.cs.statistics.StatisticValue;

public class DbOperations {

	/**
	 * The class is respoonsible of performing operations against the db
	 * 
	 */

	private Repository repository;
	
	/**
	 * Public constructor
	 * 
	 * @param repo
	 *            the type of repository to use
	 */
	public DbOperations() {
		createDriver();
		if (ApplicationParameters.getRepoType().equals("cvs")) {
			repository = new CvsOperations();
			// System.out.println("CREATED REPOSITORY CVS");
		} else if (ApplicationParameters.getRepoType().equals("svn")) {
			repository = new SvnOperations();
			// System.out.println("CREATED REPOSITORY SVN");
		}
	}

	/**
	 * This DropDbs method drops all database which are made by InstallDbs
	 * method
	 * 
	 * @throws SQLException
	 */
	public void DropDbs() throws SQLException {
		String sqlDropDbs = "DROP DATABASE IF EXISTS ";

		Connection conn = null;
		Statement stmt = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			conn.setAutoCommit(false);
			// drop each database in the evolution db dbs tables from the dbms
			for (String db : getDbInstalled()) {
				stmt = conn.createStatement();
				stmt.execute(sqlDropDbs + db);
			}
			conn.commit();
			System.out.println("All database droppedb");
		} catch (SQLException s) {
			try {
				conn.rollback();
				System.out
						.println("Unable to drop database: " + s.getMessage());
				throw s;
			} catch (SQLException se) {
			}
		} finally {
			// check if there are still some resources left open to mysql and
			// close everything
			closeStatement(stmt);
			closeConnection(conn);
		}
	}

	/**
	 * Collect the data from the databases installed and fill the evolution
	 * database Check if the table has been already filled before filling it
	 * 
	 * @throws SQLException
	 */
	private void collectEvolutionData() throws SQLException {

		String sqlInsertInto = "INSERT INTO ";
		String sqlSelect = "SELECT * FROM information_schema.";
		// String sqlSelectColumn =
		// "SELECT column_name FROM `COLUMNS` WHERE table_schema = 'information_schema' and table_name = ? ORDER BY ordinal_position";

		PreparedStatement pStmt = null;
		// Statement stmt = null;
		Connection conn = null;
		// ResultSet rs = null;
		try {

			conn = connect(ApplicationParameters.getDbevolution());
			conn.setAutoCommit(false);
			// foreach db installed and foreach table of the information schema,
			// check if the corresponding evolution table has been filled, and
			// if not fill it
			for (String db : getDbInstalled()) {

				for (String table : getEvolutionTables()) {
					// String colonna = checkDbFilled(db, table);
					int filled = checkDataCollected(db, table);
					if (filled == 0) {

						System.out.println("TABLE " + table + " DB: " + db
								+ "  NOT FILLED");

						// create the statement with the insert query

						String sql = sqlInsertInto + ApplicationParameters.getHeaderEvolutionTables() + table; // insert into
						// nome_tabella
						// get the list of the column of the table from the
						// information schema

						/*
						 * sql += "(";
						 * 
						 * conn = connect("information_schema"); pStmt =
						 * conn.prepareStatement(sqlSelectColumn);
						 * pStmt.setString(1, table); rs = pStmt.executeQuery();
						 * if(rs!=null){ while(rs.next()){
						 * //System.out.println("COLONNA " + tableIS);
						 * if(rs.isFirst()){ sql+=rs.getString(1); } else{ sql+=
						 * ","+ rs.getString(1); } } rs.close(); } conn.close();
						 * conn = null;
						 * 
						 * 
						 * sql+= ")";
						 */

						sql += "(";
						sql += sqlSelect;
						sql += table;

						sql += " WHERE ";
						sql += ApplicationParameters.getCollectionColumn(table);
						sql += " like ?)";

						System.out.println(sql);

						pStmt = conn.prepareStatement(sql);
						pStmt.setString(1, db);
						pStmt.execute();
						conn.commit();

						pStmt.close();
						pStmt = null;
					} else {
						System.out.println("TABLE " + table + " DB: " + db
								+ "  ALREADY FILLED");
					}
				}
			}
			conn.close();
			conn = null;
		} catch (SQLException sEx) {
			try {
				conn.rollback();
				System.out
						.println("An error occured during filling of the evolution database: "
								+ sEx.getMessage());
				throw sEx;
			} catch (SQLException e) {
			}
			sEx.printStackTrace();
		} finally {
			// check if there are still some resources left open to mysql and
			// close everything
			closePreparedStmt(pStmt);
			closeConnection(conn);
		}
	}

	/**
	 * Check if the data for the given db have already been collected
	 * 
	 * @param db
	 *            the name of the database to check for
	 * @param table
	 *            the name of the table to check for
	 * @return the number of rows of the given table
	 * @throws SQLException
	 */
	private int checkDataCollected(String db, String table) throws SQLException {

		// sql to retrieve the name of the column that contain a link with the
		// db installed
		// String sqlColumnSchema =
		// "SELECT column_name FROM `columns` WHERE column_name like '%schema%' AND table_schema = 'information_schema' and table_name = ? "
		// +
		// "AND ordinal_position = (SELECT MIN(ordinal_position) FROM columns WHERE table_name = ? and column_name like '%schema%')";

		// sql to check if the corresponding table of the evolution database has
		// been filled or not for the selected database
		String sqlCheckFilled = "SELECT COUNT(*) FROM ";
		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		// String colonna = null;
		int filled = 0;

		try {

			sqlCheckFilled += ApplicationParameters.getHeaderEvolutionTables()
					+ table/* .toLowerCase() */+ " WHERE "
					+ ApplicationParameters.getCollectionColumn(table) + "= ?";
			System.out.println("SQL CHECK FILLED: " + sqlCheckFilled);

			// connect to the evolution database
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sqlCheckFilled);

			pStmt.setString(1, db);
			rs = pStmt.executeQuery();

			while (rs.next()) {
				filled = rs.getInt(1);
			}
			// close all the mysql resources

			pStmt.close();
			pStmt = null;

			rs.close();
			rs = null;

			conn.close();
			conn = null;
		} catch (SQLException sEx) {
			System.out
					.println("An error occured during checking if the table into db evolution is already filled: "
							+ sEx.getMessage());
			throw sEx;
		} finally {
			// check if there are still some resources left open to mysql and
			// close everything
			closePreparedStmt(pStmt);
			closeConnection(conn);
		}
		return filled;
	}

	/**
	 * Retrieve the databases installed from the evolution database
	 * 
	 * @return Vector<String> list of the installed databases
	 * @throws SQLException
	 */
	public Vector<String> getDbInstalled() throws SQLException {

		String sql = "SELECT schema_unique_name FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_name like ? ORDER BY tstart";
		Vector<String> dbs = new Vector<String>();

		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1, ApplicationParameters.getDbbasename() + "%");
			rs = pStmt.executeQuery();

			if (rs != null) {
				while (rs.next()) {
					dbs.add(rs.getString(1));
				}
				rs.close();
			}
			conn.close();
			conn = null;

			pStmt.close();
			pStmt = null;
		} catch (SQLException sEx) {
			System.out
					.println("An error occured during retrieving the databases installed from the evolution database: "
							+ sEx.getMessage());
			throw sEx;
		} finally {
			// check if there are still some resources left open to mysql and
			// close everything
			closePreparedStmt(pStmt);
			closeConnection(conn);
		}
		return dbs;
	}
	
	
	/**
	 * TODO
	 * @return
	 * @throws SQLException
	 */
	/*public Vector<String> getDbsByTable(String table) throws SQLException {

		String sql = "SELECT table_schema FROM " + ApplicationParameters.getHeaderEvolutionTables() + "statistics s, " + ApplicationParameters.getHeaderEvolutionTables() + "dbs d WHERE s.table_schema = d.db_installed AND table_name = ? ORDER BY d.date_from";
		Vector<String> dbs = new Vector<String>();

		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1, table);
			rs = pStmt.executeQuery();

			if (rs != null) {
				while (rs.next()) {
					dbs.add(rs.getString(1));
				}
				rs.close();
			}
			conn.close();
			conn = null;

			pStmt.close();
			pStmt = null;
		} catch (SQLException sEx) {
			System.out
					.println("An error occured during retrieving the databases installed from the evolution database: "
							+ sEx.getMessage());
			throw sEx;
		} finally {
			// check if there are still some resources left open to mysql and
			// close everything
			closeConnection(conn);
			closePreparedStmt(pStmt);
			closeResultSet(rs);
		}
		return dbs;
	}*/
	

	/**
	 * Get the table of the information schema to collect the data from
	 * @return vector with the list of the tables of the information_schema
	 */
	private Vector<String> getEvolutionTables() {

		Vector<String> tables = new Vector<String>();

		Iterator it = ApplicationParameters.getCollectionTables().keySet().iterator();
		while (it.hasNext()) {
			// print all the keys of the statistics file
			String table = it.next().toString();
			tables.add(table);
		}
		return tables;
	}

	/**
	 * Create the configured tables in the evolution database
	 * 
	 * @throws SQLException
	 */
	public void CreateEvolutionTables() throws SQLException {
		Connection conn = null;
		Statement stmt = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			conn.setAutoCommit(false);
			for (String table : getEvolutionTables()) {

				// create a statement to execute the batch creation of all the
				// tables of the information schema into the db evolution
				// foreach table that need to be created into the evolution
				// database
				// create the DDL statement and create the table
				// String statement = generateCreate(table);

				// the table doesn't exist, I need to create the table in the
				// evolution db
				// if(!checkTableEvolution(table)){
				if (!tableExists(ApplicationParameters.getHeaderEvolutionTables()+ table)) {
					stmt = conn.createStatement();
					stmt.execute(generateCreate(table));
				}
			}
			conn.commit();
		} catch (SQLException se) {
			conn.rollback();
			System.out
					.println("An error occured during creating evolution tables: "
							+ se.getMessage());
			throw se;
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			// check if there are still some resources left open to mysql and
			// close everything
			closeStatement(stmt);
			closeConnection(conn);
		}
		System.out.println("All the evolution tables has been created");
		collectEvolutionData();
		System.out.println("All the evolution tables has been filled with the database data");
	}

	/**
	 * Install into the evolution DB all the revisions of the db analyzed
	 * 
	 * @throws SQLException
	 */
	public void DownloadRevisions() throws SQLException {

		String sql = "INSERT INTO " + ApplicationParameters.getEvolutionTable() + " (schema_name,svn_version,schema_unique_name,major_ordinal_version, minor_ordinal_version,tstart,tend) VALUES(?,?,?,?,?,?,?)";
		String sqlCheck = "SELECT * FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_name = ? AND svn_version= ?";
		String sqlMajorVersion = "SELECT MAX(major_ordinal_version) FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_name like ? ";
		
		Vector<DbVersion> revisions = null;

		// connection variables
		Connection conn = null;
		PreparedStatement pStmt = null;
		PreparedStatement pStmtCheck = null;
		PreparedStatement pStmtMajor = null;
		ResultSet rs = null;
		ResultSet rsMajor = null;
		int maxMajorVersion = 0;
		revisions = repository.GetRevisions();

		try {

			createDriver();
			// establish a connection to the dbms
			conn = connect(ApplicationParameters.getDbevolution());

			pStmt = conn.prepareStatement(sql);
			pStmtCheck = conn.prepareStatement(sqlCheck);
			pStmtMajor = conn.prepareStatement(sqlMajorVersion);
			
			pStmtMajor.setString(1, ApplicationParameters.getDbbasename()+"%");
			rsMajor = pStmtMajor.executeQuery();
			
			while (rsMajor.next()) {
				maxMajorVersion = rsMajor.getInt(1);
			}
			
			if(ApplicationParameters.getRepoType().equals("cvs")){
				maxMajorVersion +=  revisions.size();
			}
			
			// disabel the auto-commit property of the connection, I'll call the
			// commit method after all the operations have been executed on the DB
			conn.setAutoCommit(false);

			// execute all the insert query;
			// if one fails the transaction is rolled back
			for (int i = 0; i < revisions.size(); i++) {

				// check if there is already a tuple in the VERSIONS table with the same name values
				pStmtCheck.setString(1, revisions.get(i).get_dbName());
				pStmtCheck.setString(2, revisions.get(i).get_dbVersion());
				rs = pStmtCheck.executeQuery();

				// the result set is empty
				// add the new revision to the databases table
				if (!rs.first()) {
					pStmt.setString(1, revisions.get(i).get_dbName());
					pStmt.setString(2, revisions.get(i).get_dbVersion());
					pStmt.setString(3, revisions.get(i).get_dbName()+ Utils.cleanRevisionNumber(revisions.get(i).get_dbVersion()));
					if(ApplicationParameters.getRepoType().equals("cvs")){
						pStmt.setInt(4, maxMajorVersion--);
					}
					else{
						pStmt.setInt(4, ++maxMajorVersion);
					}
					pStmt.setInt(5, 0);
					pStmt.setTimestamp(6, new java.sql.Timestamp(revisions.get(i).get_dateFrom().getTime()));
					pStmt.setTimestamp(7, new java.sql.Timestamp(revisions.get(i).get_dateTo().getTime()));
					pStmt.execute();
					System.out.println("DB:" + revisions.get(i).get_dbName()+ revisions.get(i).get_dbVersion() + "INSERTED");
				}
			}
			conn.commit();
		} catch (SQLException e) {
			try {
				conn.rollback();
			} catch (SQLException error) {
			}
			throw e;
		} finally {

			// check if there are still some resources left open to mysql and
			// close everything
			closePreparedStmt(pStmt);
			closePreparedStmt(pStmtCheck);
			closeResultSet(rs);
			closeResultSet(rsMajor);
			closeConnection(conn);
		}

	}

	/**
	 * Install all the revision of the schema
	 * 
	 * @param revision
	 */
	public void InstallDbs() throws SQLException {

		String pathToSchema = ApplicationParameters.getPathtoschema();
		String sqlCreateDb = "CREATE DATABASE IF NOT EXISTS ";
		String sqlGetSchemas = "SELECT schema_unique_name,svn_version, state FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_name = ?";
		String sqlCheckIS = "SELECT COUNT(*) FROM SCHEMATA WHERE SCHEMA_NAME = ?";
		String sqlStateUpd = "UPDATE " + ApplicationParameters.getEvolutionTable() + " SET state = 1 WHERE schema_unique_name =?";
		String sqlMysqlErr = "UPDATE " + ApplicationParameters.getEvolutionTable() + " SET error_code = ?, error_msg = ? WHERE schema_unique_name=?";
		String dbName = ApplicationParameters.getDbbasename();
		String sqlExt = ".sql";

		PreparedStatement pStmt = null;
		Statement stmt = null;
		Connection conn = null;
		Connection connS = null;
		ResultSet rs = null;

		int rowCount = 0;
		
		// GET ALL THE DBS TO INSTALL FROM THE EVOLUTION DB
		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sqlGetSchemas);
			pStmt.setString(1, dbName);
			rs = pStmt.executeQuery();

			// FOREACH DB CHECK WHETER OR NOT IS INSTALLED
			// DOUBLE CHECK: - CHECK THE FLAG STATUS EQUALS TO 0
			// - CHECK THE INFORMATION SCHEMA
			// USED FOR APPLICATION TESTING ON NEW SYSTEM
			while (rs.next()) {
				//String name = dbName + Utils.cleanRevisionNumber(rs.getString("version"));
				//retrieve the values from the result set
				String name = rs.getString("schema_unique_name");
				String version = rs.getString("svn_version");
				int state = rs.getInt("state"); 
				System.out.println("DB NAME:" + name);

				//FIRST CHECK
				//the state's flag is equal to 0, the database is not installed
				if (state == 0) {
					//SECOND CHECK
					//check the database name against the information schema
					connS = connect("information_schema");
					pStmt = connS.prepareStatement(sqlCheckIS);
					pStmt.setString(1, name);
					ResultSet rsCount = pStmt.executeQuery();
					while (rsCount.next()) {
						rowCount = rsCount.getInt(1);
					}
					rsCount.close();
					connS.close();
					
					if (rowCount == 0) {
						// THE DB MUST BE INSTALLED
						// CHECK IF THE SCHEMA IS AVAILABLE
						// IF NOT AVAILABLE, DOWNLOAD IT
						File f = new File(pathToSchema + version + sqlExt);
						File fdir = new File(ApplicationParameters
								.getPathtoschema());
						if (!fdir.exists()) {
							try {
								System.out
										.println("Creating directory schema files...");
								boolean dirCreated = fdir.mkdir();
								if (dirCreated) {
									System.out
											.println("Directory schema path created");
								}
							} catch (SecurityException sEx) {
								System.out
										.println("The user doesn't have the privileges to create the directory schema");
								System.exit(1);
							}
						}
						System.out.println("Checking the schema: "
								+ pathToSchema + version + sqlExt);
						if (!f.exists()) {
							try {
								repository.DownloadRevision(version);
								System.out.println("Downloading the schema: "
										+ version);
							} catch (NumberFormatException ne) {
								System.out.println("NumberFormatException: "
										+ ne.getMessage());
								continue;
							}
						}
						Vector<Query> queries = new Vector<Query>();
						SqlScript script = new SqlScript((pathToSchema
								+ version + sqlExt));
						try {
							queries = script.loadScript();
						} catch (IOException e) {
							System.out
									.println("Error whil loading the schema: "
											+ e.getMessage());
							continue;
						}
						
						// INSTALL THE DB
						try {
							connS = connect("information_schema");
							stmt = connS.createStatement();
							stmt.executeUpdate(sqlCreateDb + name);
							connS.close();
							System.out.println("THE DB " + name
									+ " WAS SUCCESSFULLY CREATED");
							//the database has been successfully created, install the tables from the script file
							try {
								connS = connect(name);
								/*
								 * TRY TO CHANGE THE WAY THE MYSQL COMMAND ARE ISSUED AGAINST THE SERVER
								 * USING A BATCH STATEMENT EXECUTION TO AVOID ERRORS WITH FOREIGN KEYS
								 * 
								 * for (int i = 0; i < queries.size(); i++) {
									// System.out.println(queries.get(i).getQuerytext());
									stmt = connS.createStatement();
									stmt.executeUpdate(queries.get(i)
											.getQuerytext());

								}*/
								stmt = connS.createStatement();
								stmt.addBatch("SET FOREIGN_KEY_CHECKS=0;");
								for (int i = 0; i < queries.size(); i++) {
									// System.out.println(queries.get(i).getQuerytext());
									stmt.addBatch(queries.get(i).getQuerytext());
								}
								stmt.addBatch("SET FOREIGN_KEY_CHECKS=1;");
								stmt.executeBatch();
								connS.close();
								try {
									//IF no errors occurred, update the state's flag into the VERSIONS table
									connS = connect(ApplicationParameters
											.getDbevolution());
									connS.setAutoCommit(false);
									pStmt = connS.prepareStatement(sqlStateUpd);
									pStmt.setString(1, name);
									pStmt.executeUpdate();
									connS.commit();
								} catch (SQLException e) {
									connS.rollback();
									System.out
											.println("AN ERROR OCCUPRED DURING THE UPDATE OF THE STATUS FLAG: "
													+ name
													+ " MESSAGE: "
													+ e.getMessage());
								}
							} catch (SQLException se) {
								System.out
										.println("AN ERROR IS OCCURED DURING THE INSTALLATION OF THE SCHEMA: "
												+ name
												+ " MESSAGE: "
												+ se.getMessage());
								try {
									//UPDATE THE ERROR_CODE AND ERROR_MSG USING THE SERVER STATUS'S INFORMATIONS
									connS = connect(ApplicationParameters
											.getDbevolution());
									connS.setAutoCommit(false);
									pStmt = connS.prepareStatement(sqlMysqlErr);
									pStmt.setString(1, Integer.toString(se
											.getErrorCode()));
									// the field is unsed at the moment due to
									// space problem
									pStmt.setString(2, "");
									pStmt.setString(3, name);
									pStmt.executeUpdate();
									connS.commit();
									connS.close();
								} catch (SQLException se2) {
									throw se2;
								}
							}

						} catch (SQLException e) {
							System.out
									.println("ERROR DURING THE CREATION OF THE DB: "
											+ name
											+ " MESSAGE: "
											+ e.getMessage());
						}/**/
					} else {
						System.out.println("DB: " + name
								+ " IS ALREADY INSTALLED");
					}
				} else {
					System.out.println("DB: " + name + " IS ALREADY INSTALLED");
				}
			}
			rs.close();
			conn.close();

		} catch (SQLException se) {
			System.out.println("ERROR WHILE GETTING THE DB VERSIONS: "
					+ se.getMessage());
			se.printStackTrace();
			System.exit(1);
		}
		finally{
			closeStatement(stmt);
			closePreparedStmt(pStmt);
			closeConnection(connS);
			closeConnection(conn);
		}

	}

	/**
	 * Generate the create statement to create the specified table.
	 * 
	 * @param table
	 *            The table to generate a create statement for.
	 * @return The create table statement.
	 * @throws DatabaseException
	 *             If a database error occurs.
	 */
	public String generateCreate(String table) throws SQLException {
		StringBuffer result = new StringBuffer();
		Connection conn = null;
		ResultSet rs = null;
		Statement stmt = null;
		try {
			StringBuffer sql = new StringBuffer();

			conn = connect("information_schema");

			sql.append("SELECT * FROM ");
			sql.append(table);
			stmt = conn.createStatement();
			rs = stmt.executeQuery(sql.toString());
			ResultSetMetaData md = rs.getMetaData();

			result.append("CREATE TABLE ");
			result.append(ApplicationParameters.getHeaderEvolutionTables()
					+ table);
			result.append(" ( ");

			for (int i = 1; i <= md.getColumnCount(); i++) {
				if (i != 1)
					result.append(',');
				result.append(md.getColumnName(i));
				result.append(' ');

				String type = processType(md.getColumnTypeName(i), md
						.getPrecision(i));
				result.append(type);

				if (md.getPrecision(i) < 65535) {
					if (!this.isDateTime(md.getColumnType(i))) {
						result.append('(');
						result.append(md.getPrecision(i));
						if (md.getScale(i) > 0) {
							result.append(',');
							result.append(md.getScale(i));
						}
						result.append(") ");
					} else {
						result.append(' ');
					}
				} else {
					result.append(' ');
				}
				if (this.isNumeric(md.getColumnType(i))) {
					if (!md.isSigned(i))
						result.append("UNSIGNED ");
				}

				if (md.isNullable(i) == ResultSetMetaData.columnNoNulls)
					result.append("NOT NULL ");
				else
					result.append("NULL ");
				if (md.isAutoIncrement(i))
					result.append(" auto_increment");
			}
			/*
			 * DatabaseMetaData dbm = conn.getMetaData(); ResultSet primary =
			 * dbm.getPrimaryKeys(null, null, table); boolean first = true;
			 * while (primary.next()) { if (first) { first = false;
			 * result.append(','); result.append("PRIMARY KEY("); } else
			 * result.append(",");
			 * 
			 * result.append(primary.getString("COLUMN_NAME")); }
			 * 
			 * if (!first) result.append(')');
			 */
			result.append(" ); ");
		} catch (SQLException sEx) {
			System.out
					.println("An error is occured during generating the CREATE statement: "
							+ sEx.getMessage());
			throw sEx;
		} finally {

		}
		System.out.println(result.toString());
		return result.toString();
	}

	/**
	 * Check to see if the specified type is numeric.
	 * 
	 * @param type
	 *            The type to check.
	 * @return Returns true if the type is numeric.
	 */
	public boolean isNumeric(int type) {
		if (type == java.sql.Types.BIGINT || type == java.sql.Types.DECIMAL
				|| type == java.sql.Types.DOUBLE
				|| type == java.sql.Types.FLOAT
				|| type == java.sql.Types.INTEGER
				|| type == java.sql.Types.NUMERIC
				|| type == java.sql.Types.SMALLINT
				|| type == java.sql.Types.TINYINT)
			return true;
		else
			return false;

	}

	/**
	 * Check to see if the specified type is datatype or similar
	 * 
	 * @param type
	 *            The type to check
	 * @return true if the type is datatype
	 */
	public boolean isDateTime(int type) {
		if (type == java.sql.Types.DATE || type == java.sql.Types.TIME
				|| type == java.sql.Types.TIMESTAMP)
			return true;
		else
			return false;

	}

	/**
	 * process a database type. Sometimes database types are
	 * not reported exactly as they need to be for proper syntax. This method
	 * corrects the database type and size.
	 * 
	 * @param type
	 *            The type reported
	 * @param i
	 *            The size of this column
	 * @return The properly formatted type, for this database
	 */
	public String processType(String type, int size) {
		String usigned = "UNSIGNED";
		int i = type.indexOf(usigned);
		if (i != -1)
			type = type.substring(0, i) + type.substring(i + usigned.length());

		if (type.equalsIgnoreCase("varchar") && (size == 65535))
			type = "TEXT";
		return type.trim();
	}

	/**
	 * Determine if a table exists.
	 * 
	 * @param table
	 *            The name of the table.
	 * @return True if the table exists.
	 * @throws SQLException a database error occurred.
	 */
	public boolean tableExists(String table) throws SQLException {
		boolean result = false;
		ResultSet rs = null;
		Connection conn = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			DatabaseMetaData dbm = conn.getMetaData();

			String types[] = { "TABLE" };
			rs = dbm.getTables(null, null, table, types);
			result = rs.next();
			rs.close();
		} catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
		}
		return result;
	}

	
	
	/**
	 * Return the name of the tables for the specified database 
	 * @param db name
	 * @return Vector of tables name
	 * @throws SQLException
	 */  
	public Vector<String> getTables(String db) throws SQLException {

		Vector<String> tables = new Vector<String>();
		String sql = "SELECT DISTINCT TABLE_NAME FROM "
				+ ApplicationParameters.getHeaderEvolutionTables()
				+ "TABLES WHERE TABLE_SCHEMA LIKE ?";
		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1, db);

			rs = pStmt.executeQuery();
			if (rs != null) {
				while (rs.next()) {
					tables.add(rs.getString(1));
				}
				rs.close();
				rs = null;
			}
			conn.close();
			conn = null;
			
			pStmt.close();
			pStmt = null;
		} 
		catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
			closePreparedStmt(pStmt);
		}
		return tables;
	}

	

	
	/**
	 * TODO
	 * @param tableName
	 * @return
	 * @throws SQLException
	 */
	/*public Vector<String> getLiveRevisions(String tableName) throws SQLException {

		Vector<String> revisions = new Vector<String>();
		String sql = "SELECT DISTINCT(table_schema) FROM " + ApplicationParameters.getHeaderEvolutionTables() + "tables t,"+ ApplicationParameters.getHeaderEvolutionTables() +
				"dbs d WHERE t.table_schema = d.db_installed and table_name like ? ORDER BY d.date_from";
		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1, tableName);

			rs = pStmt.executeQuery();
			if (rs != null) {
				while (rs.next()) {
					revisions.add(rs.getString(1));
				}
				rs.close();
				rs = null;
			}
			conn.close();
			conn = null;
			
			pStmt.close();
			pStmt = null;
		} 
		catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
			closePreparedStmt(pStmt);
		}
		return revisions;
	}/

	
	/**
	 * TODO
	 * @param tableName
	 * @return
	 * @throws SQLException
	 */
	/*public Vector<String> getColumnsName(String tableSchema, String tableName) throws SQLException {

		Vector<String> columnsName = new Vector<String>();
		String sql = "SELECT column_name FROM " + ApplicationParameters.getHeaderEvolutionTables() + "columns WHERE table_schema like ? AND table_name = ?";
		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1, tableSchema);
			pStmt.setString(2, tableName);

			rs = pStmt.executeQuery();
			if (rs != null) {
				while (rs.next()) {
					columnsName.add(rs.getString(1));
				}
				rs.close();
				rs = null;
			}
			conn.close();
			conn = null;
			
			pStmt.close();
			pStmt = null;
		} 
		catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
			closePreparedStmt(pStmt);
		}
		return columnsName;
	}*/
	
	
	// *****************************************************************************************************//
	// //
	// //
	// STATISTICS METHOD //
	// //
	// //
	// //
	// *****************************************************************************************************//

	/**
	 * Get the count of the columns for every database
	 * @return a vector containing the count of the table for the selected schema
	 */
	public Vector<StatisticValue> GetStatCount(String schema, String sql)throws SQLException {

		Vector<StatisticValue> chartValue = new Vector<StatisticValue>();

		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1, schema + "%");

			rs = pStmt.executeQuery();
			if (rs != null) {
				while (rs.next()) {
					StatisticValue statColCount = new StatisticValue();
					statColCount.setX(rs.getString(1));
					statColCount.setY(rs.getInt(2));
					chartValue.add(statColCount);
				}
				rs.close();
				rs = null;
			}
			conn.close();
			conn = null;
			
			pStmt.close();
			pStmt = null;
		} catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
			closePreparedStmt(pStmt);
		}
		return chartValue;
	}
	
	
	
	/**
	 * Get the count of the columns for every database
	 * @return a vector containing the count of the table for the selected schema
	 */
	public Vector<StatisticValue> getRevisionLifetime(Statistic stat)throws SQLException {

		Vector<StatisticValue> chartValue = new Vector<StatisticValue>();

		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(stat.getSqlStmt());
			pStmt.setString(1, ApplicationParameters.getDbbasename() + "%");
			pStmt.setString(2, ApplicationParameters.getDbbasename() + "%");

			rs = pStmt.executeQuery();
			if (rs != null) {
				while (rs.next()) {
					StatisticValue statColCount = new StatisticValue();
					statColCount.setX(rs.getString(1));
					statColCount.setY(rs.getInt(2));
					chartValue.add(statColCount);
				}
				rs.close();
				rs = null;
			}
			conn.close();
			conn = null;
			
			pStmt.close();
			pStmt = null;
		} catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
			closePreparedStmt(pStmt);
		}
		return chartValue;
	}
	
	
	
	/**
	 * Get the number of month each revision was live
	 * @param schema 
	 * @param sql
	 * @return
	 * @throws SQLException
	 */
	public Vector<MonthRevision> getMonthRevisions(String schema, String sql)throws SQLException {

		Vector<MonthRevision> chartValue = new Vector<MonthRevision>();

		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1, schema + "%");

			rs = pStmt.executeQuery();
			if (rs != null) {
				while (rs.next()) {
					MonthRevision month = new MonthRevision();
					month.setMonth(rs.getString(2)); 
					month.setYear(rs.getString(1));
					month.setRevisions(rs.getInt(3));
					chartValue.add(month);
				}
				rs.close();
				rs = null;
			}
			conn.close();
			conn = null;
			
			pStmt.close();
			pStmt = null;
		} catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
			closePreparedStmt(pStmt);
		}
		return chartValue;
	}

	
	/**
	 * TODO
	 */  
	public Vector<String> getTablesWithKeys(String db) throws SQLException {

		Vector<String> tables = new Vector<String>();
		String sql = "SELECT DISTINCT table_name from " + ApplicationParameters.getHeaderEvolutionTables() + "TABLE_CONSTRAINTS WHERE table_schema like ? AND constraint_name like 'PRIMARY'";
		
		String[] params = new String[]{db};
		
		Vector<String[]> result = executeStatement(sql, params);
		
		for(int i=0;i<result.size();i++){
			tables.add( ((String[]) result.get(i) )[0]); 
		}
		
		return tables;
	}
	
	
	
	/**
	 * TODO
	 * @param db
	 * @return
	 * @throws SQLException
	 */
	public Vector<String[]> getTablesWithForeignKeys(String db) throws SQLException {

		String sql = "SELECT table_name, constraint_name FROM " + ApplicationParameters.getHeaderEvolutionTables() + "TABLE_CONSTRAINTS WHERE table_schema like ? " +
				"AND constraint_name like 'FOREIGN%' ORDER BY constraint_schema";
		
		String[] params = new String[]{db};
		
		return executeStatement(sql, params);
	}
	
	
	
	/**
	 * Get the name of the tables with index;
	 * use the table table_privileges to perform a comparison and discard those tuples corresponding to primary keys and foreign keys
	 * @param db
	 * @return
	 * @throws SQLException
	 */
	public Vector<String[]> getTablesWithIndexes(String db) throws SQLException {

		String sql = "SELECT DISTINCT s.table_name, index_name FROM " + ApplicationParameters.getHeaderEvolutionTables() + "STATISTICS AS s LEFT JOIN " + ApplicationParameters.getHeaderEvolutionTables() + "TABLE_CONSTRAINTS AS c " +
			"ON s.table_schema = c.constraint_schema AND s.index_name = c.constraint_name AND s.table_name = c.table_name " +
			"WHERE s.table_schema LIKE ? AND ( c.constraint_type NOT LIKE 'PRIMARY%' AND c.constraint_type NOT LIKE 'FOREIGN%' " +
			"OR constraint_type IS NULL ) ORDER BY s.table_name";

		String[] params = new String[]{db};
		
		return executeStatement(sql, params);
	}
	
	
	/**
	 * Get the name of the columns composing the index for all the revision of a database
	 * @param dbName
	 * @param tableName
	 * @param indexName
	 * @return
	 * @throws SQLException
	 */
	public Vector<String[]> getColumnsInIndex(String dbName,String tableName,String indexName) throws SQLException {

		String sql = "SELECT table_schema, column_name FROM " + ApplicationParameters.getHeaderEvolutionTables() + "STATISTICS AS s WHERE s.table_schema LIKE ? " +
				"AND s.table_name like ? AND s.index_name like ?";

		String[] params = new String[]{dbName, tableName, indexName};
		
		return executeStatement(sql, params);
		
	}
	
	
	/**
	 * Get the name of the columns composing the foreign key for all the revision of a database
	 * @param dbName
	 * @param tableName
	 * @param indexName
	 * @return
	 * @throws SQLException
	 */
	public Vector<String[]> getColumnsInForeign(String dbName,String tableName,String indexName) throws SQLException {

		String sql = "SELECT table_schema, column_name FROM " + ApplicationParameters.getHeaderEvolutionTables() + "STATISTICS AS s WHERE s.table_schema LIKE ? " +
				"AND s.table_name like ? AND s.index_name like ?";

		String[] params = new String[]{dbName, tableName, indexName};
		
		return executeStatement(sql, params);
		
	}
	
	
	/**
	 * Get the name of the columns
	 * @param dbName the database name
	 * @param tableName the table name
	 * @return column's name for each revision of the table
	 * @throws SQLException
	 */
	public Vector<String[]> getColumnsInTable(String dbName,String tableName) throws SQLException {

		String sql = "SELECT table_schema, column_name FROM " + ApplicationParameters.getHeaderEvolutionTables() + "COLUMNS c, " + ApplicationParameters.getEvolutionTable() + " d WHERE d.schema_unique_name = c.table_schema " +
				"AND table_schema like ? AND table_name like ? ORDER BY d.tstart";

		String[] params = new String[]{dbName, tableName};
		
		return executeStatement(sql, params);
		
	}
	

	
	/**
	 * TODO
	 * @param table
	 * @param db
	 * @return
	 * @throws SQLException
	 */
	public Vector<String[]> getColumnsInPrimary(String table, String db) throws SQLException {

		String sql = "SELECT table_schema, column_name FROM " + ApplicationParameters.getHeaderEvolutionTables() + "KEY_COLUMN_USAGE s," 
				+ ApplicationParameters.getEvolutionTable() + " d WHERE d.schema_unique_name =  s.table_schema " +
				"AND table_name = ? AND table_schema like ? AND constraint_name = ? " +
				"ORDER BY d.tstart";
		String[] params = new String[]{table, db, "PRIMARY"};
		return this.executeStatement(sql, params);
	}
	
	
	
	/**
	 * TODO
	 * @param table
	 * @param db
	 * @return
	 * @throws SQLException
	 */
	public Vector<String[]> getIndexInTable(String table, String db) throws SQLException {

		String sql = "SELECT DISTINCT s.table_name, index_name FROM STATISTICS AS s LEFT JOIN " + ApplicationParameters.getHeaderEvolutionTables() + "TABLE_CONSTRAINTS AS c " +
				"ON s.table_schema = c.constraint_schema AND s.index_name = c.constraint_name AND s.table_name = c.table_name " +
				"WHERE s.table_schema LIKE 'ensembl%' AND ( c.constraint_type NOT LIKE 'PRIMARY%' AND c.constraint_type NOT LIKE 'FOREIGN%' " +
				"OR constraint_type IS NULL )";
		
		String[] params = new String[]{table, db, "PRIMARY"};
		return this.executeStatement(sql, params);
	}
	
	
	
	
	/**
	 * Return the name of the database installed with the minimum date with respect to the parameters passed
	 * The first database where the table occured
	 * @param tableSchema
	 * @param tableName
	 * @return
	 * @throws SQLException
	 */
	public String getSchemaByDate(String tableSchema, String tableName,String date) throws SQLException{
		
		String sql = "SELECT schema_unique_name FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_name = ? and tstart = ";
		String db = null;
		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		if(date.equalsIgnoreCase("min"))
			sql+="(SELECT MIN(tstart) FROM " +  ApplicationParameters.getEvolutionTable() + " d, " +  ApplicationParameters.getHeaderEvolutionTables() + "TABLES t where d.schema_unique_name = t.table_schema and schema_name=? and t.table_name = ?)";
		else
			sql+="(SELECT MAX(tstart) FROM " +  ApplicationParameters.getEvolutionTable() + " d, " +  ApplicationParameters.getHeaderEvolutionTables() + "TABLES t where d.schema_unique_name = t.table_schema and schema_name=? and t.table_name = ?)";
					
		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			
			pStmt.setString(1, tableSchema);
			pStmt.setString(2, tableSchema);
			pStmt.setString(3, tableName);
			
			rs = pStmt.executeQuery();
			if (rs != null) {
				rs.next();
				db = rs.getString(1);
			}
			closeConnection(conn);
			closePreparedStmt(pStmt);
		} 
		catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
			closePreparedStmt(pStmt);
		}
		return db;
	}
	
	
	
	
	
	
	// *****************************************************************************************************//
	// //
	// //
	// CONNECTION METHOD //
	// //
	// //
	// //
	// *****************************************************************************************************//

	/**
	 * Open a connection towards the selected database and return a connection
	 * object
	 */
	public static Connection connect(String dbname) {
		Connection conn = null;
		try {
			conn = (Connection) DriverManager.getConnection(
					ApplicationParameters.getDbbaseuri() + dbname,
					ApplicationParameters.getUser(), ApplicationParameters
							.getPass());
			// Do something with the Connection

		} catch (SQLException ex) {
			// handle any errors
			System.out.println("SQLException: " + ex.getMessage());
			System.out.println("SQLState: " + ex.getSQLState());
			System.out.println("VendorError: " + ex.getErrorCode());
			System.exit(1);
		}
		return conn;
	}

	/**
	 * Create the driver for the connection
	 * 
	 */
	private void createDriver() {
		try {
			// The newInstance() call is a work around for some
			// broken Java implementations
			Class.forName("com.mysql.jdbc.Driver").newInstance();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	
	/**
	 * Close a ResulSet object
	 * @param rs
	 */
	private void closeResultSet(ResultSet rs){
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
			}
		}
	}
	
	/**
	 * Close a Connection object
	 * @param conn
	 */
	private void closeConnection(Connection conn){
		if (conn != null) {
			try {
				conn.close();
			}
			catch (SQLException e) {
			}
		}
	}
	
	/**
	 * Close a PreparedStatement object
	 * @param pStmt
	 */
	private void closePreparedStmt(PreparedStatement pStmt){
		if (pStmt != null) {
			try {
				pStmt.close();
			} catch (SQLException sqlex) {
			}
			pStmt = null;
		}
	}
	
	/**
	 * Close a Statement object
	 * @param stmt
	 */
	private void closeStatement(Statement stmt){
		if (stmt != null) {
			try {
				stmt.close();
			} catch (SQLException sqlex) {
			}
			stmt = null;
		}
	}


	/**
	 * TODO
	 * @param sql
	 * @param params
	 * @return
	 * @throws SQLException
	 */
	public Vector<String[]> executeStatement(String sql, String[] params) throws SQLException{
		
		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;
		int colonne = 0;
		String [] record;
		
		Vector<String[]> result;

		try {
			conn = connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			
			for(int i=1;i<=params.length; i++){
				pStmt.setString(i, params[i-1]);
			}
			
			rs = pStmt.executeQuery();
			result = new Vector<String[]>();
			ResultSetMetaData rsmd = rs.getMetaData();
			colonne = rsmd.getColumnCount();

 			while(rs.next()) {   // Creo il vettore risultato scorrendo tutto il ResultSet
 				record = new String[colonne];
 				for (int i=0; i<colonne; i++){ 
 					record[i] = rs.getString(i+1);
 				}
 				result.add( (String[]) record.clone() );
 			}
			closeConnection(conn);
			closePreparedStmt(pStmt);
		} 
		catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
			closePreparedStmt(pStmt);
		}
		return result;
	}

	
	/**
	 * TODO
	 * @return
	 * @throws SQLException
	 */
	public String getDatabaseFirstRelease() throws SQLException {
		String sql = "SELECT DATE(tstart) FROM " +  ApplicationParameters.getEvolutionTable() + " WHERE schema_name LIKE ? AND tstart = (SELECT MIN(tstart) FROM " +  ApplicationParameters.getEvolutionTable() + " WHERE schema_name LIKE ? )"; 
		String[] params = {ApplicationParameters.getDbbasename(), ApplicationParameters.getDbbasename()};
		Vector<String[]> result;
		result = executeStatement(sql, params);
		return ( (String[])result.get(0))[0];
	}

	
	/**
	 * TODO
	 * @return
	 * @throws SQLException
	 */
	public String getDatabaseLastRelease() throws SQLException {
		String sql = "SELECT DATE(tstart) FROM " +  ApplicationParameters.getEvolutionTable() + " WHERE schema_name LIKE ? AND tstart = (SELECT MAX(tstart) FROM " +  ApplicationParameters.getEvolutionTable() + " WHERE schema_name LIKE ? )"; 
		String[] params = {ApplicationParameters.getDbbasename(), ApplicationParameters.getDbbasename()};
		Vector<String[]> result = executeStatement(sql, params);
		return ( (String[])result.get(0))[0];
	}

	
	/**
	 * TODO
	 * @return
	 * @throws SQLException
	 */
	public int getDatabaseInitialSize() throws SQLException {
		String sql = "SELECT COUNT(*) FROM " +  ApplicationParameters.getHeaderEvolutionTables() + "TABLES WHERE table_schema in (SELECT schema_unique_name FROM " +  ApplicationParameters.getEvolutionTable() + " WHERE schema_name LIKE ? AND tstart = (SELECT MIN(tstart) FROM " +  ApplicationParameters.getEvolutionTable() + " WHERE schema_name LIKE ? ))"; 
		String[] params = {ApplicationParameters.getDbbasename(), ApplicationParameters.getDbbasename()};
		Vector<String[]> result = executeStatement(sql, params);
		return Integer.valueOf(( (String[])result.get(0))[0]);
	}

	
	/**
	 * TODO
	 * @return
	 * @throws SQLException
	 */
	public int getDatabaseEndSize() throws SQLException {
		String sql = "SELECT COUNT(*) FROM " +  ApplicationParameters.getHeaderEvolutionTables() + "TABLES WHERE table_schema in (SELECT schema_unique_name FROM " +  ApplicationParameters.getEvolutionTable() + " WHERE schema_name LIKE ? AND tstart = (SELECT MAX(tstart) FROM " +  ApplicationParameters.getEvolutionTable() + " WHERE schema_name LIKE ? ))"; 
		String[] params = {ApplicationParameters.getDbbasename(), ApplicationParameters.getDbbasename()};
		Vector<String[]> result = executeStatement(sql, params);
		return Integer.valueOf(( (String[])result.get(0))[0]);
	}
	


}
