package edu.ucla.cs.dbevolution;

import it.polimi.elet.cautils.configurator.CACmdLineParser;
import it.polimi.elet.cautils.configurator.CAConfigFileParser;
import it.polimi.elet.cautils.configurator.CAConfiguration;
import it.polimi.elet.cautils.configurator.CAConfigurator;

import java.sql.SQLException;
import java.util.Iterator;

import edu.ucla.cs.statistics.StatisticsGenerator;
import edu.ucla.cs.statistics.StatisticsParameters;

/**
 * @author fangar
 *
 */
public class MenuApplication {

	/**
	 * This is the menu of the application you can use to control all the functionality provided
	 * Read the command line parameters and set the application parameter
	 * @param args
	 */
	public static void main(String[] args){
		
		CAConfigurator c = null;
		CAConfiguration conf = null;
			
		//parse the command line argument and set the application parameter
		//USE CAConfigurator to load configuration from inputline
		CACmdLineParser cmdParser = null;
		/*try {
			par = c.getInstance().getCmdLineParserInstance();
		} catch (InstantiationException e1) {
			e1.printStackTrace();
		}*/
		try {
			cmdParser = CACmdLineParser.getInstance();
		} 
		catch (InstantiationException e) {
			System.out.println("Unable to parse the command line arguments");
			e.printStackTrace();
			System.exit(1);
		}
		
		//set the configuration parameter that can be read from the command line
		cmdParser.addStringParameter("repositoryType");
		cmdParser.addStringParameter("pathtoschema");
		cmdParser.addStringParameter("dbbasename");
		//SVN
		cmdParser.addStringParameter("svnurl");
		cmdParser.addStringParameter("svnuser");
		cmdParser.addStringParameter("svnpass");
		//CVS
		cmdParser.addStringParameter("cvsRoot");
		cmdParser.addStringParameter("cvsModule");
		cmdParser.addStringParameter("cvsSaveSchemaPath");
		cmdParser.addStringParameter("cvsLocalPath");
		//OPERATIONS
		cmdParser.addStringParameter("collect");
		cmdParser.addStringParameter("download");
		cmdParser.addStringParameter("drop");
		cmdParser.addStringParameter("global");
		cmdParser.addStringParameter("install");
		
				
		if(args.length<1){
			System.out.println("No command line option set, the configuration file must be present\n");
			System.exit(1);
		}
		try{ 
			conf = cmdParser.parse(args);
			String configpath = (String) conf.getParameterValue("config");
			//if the config path is set and equals to a configuration file, read and parse the configuration file
			if(configpath != null){
				CAConfigFileParser filepar = c.getConfigFileParserInstance();
				conf = filepar.parse(configpath);
			}
		
			//parse again the configuration to override the parameters from the command line
			conf = cmdParser.parse(args);
		
			overrideFromCommandLine(conf);
	
			//reads the application parameter
			ApplicationParameters.loadConfiguration(conf);
			setCollectionTableInfo();
			inizializeStatisticsParameters();
		}
		catch(NullPointerException nE){
			System.out.println("All the parameters in the configuration file must have a value\n");
			System.exit(1);
		}
		executeFunction();
	}
	
	
	public static void inizializeParameters(String configpath){

		CAConfiguration conf = null;
		
		//if the file configuration path is set and equals to a configuration file, read and parse the configuration file
		if(configpath != null){
			CAConfigFileParser filepar = CAConfigurator.getConfigFileParserInstance();
			conf = filepar.parse(configpath);
		}
		
		//set the application parameter
		try{
			ApplicationParameters.loadConfiguration(conf);
			setCollectionTableInfo();
			inizializeStatisticsParameters();
		}
		catch(NullPointerException nEx){
			System.out.println("Unable to start: set all the parameters of the configuratino file");
			System.exit(1);
		}
	}
	
	
	
	public static void inizializeStatisticsParameters(){

		CAConfiguration confStatistics = null;
		
		//if the file configuration path is set and equals to a configuration file, read and parse the configuration file
		if(ApplicationParameters.getStatisticsFile() != null){
			CAConfigFileParser filepar = CAConfigurator.getConfigFileParserInstance();
			confStatistics = filepar.parse(ApplicationParameters.getStatisticsFile());
		}
		//set the application parameter
		StatisticsParameters.loadConfiguration(confStatistics);
	}
	
	/**
	 * TODO
	 */
	public static void executeFunction(){
		DbOperations dbOp = new DbOperations();
		
		try{
			if(ApplicationParameters.getParameterValue("global")){ 
				System.out.println("ALL THE OPERATION WILL BE PERFORMED");
				//check all the necessary parameter are set
				dbOp.DropDbs();
				dbOp.DownloadRevisions();
				dbOp.InstallDbs();
				dbOp.CreateEvolutionTables();
				dbOp.DropDbs();
			}
			else{
				if(ApplicationParameters.getParameterValue("drop")){
					//check all the necessary parameter are set
					System.out.println("DROP");
					dbOp.DropDbs();
				}
				if(ApplicationParameters.getParameterValue("download")){
					//check all the necessary parameter are set
					dbOp.DownloadRevisions();
				}
				if(ApplicationParameters.getParameterValue("install")){
					//check all the necessary parameter are set			
					System.out.println("INSTALL");
						dbOp.InstallDbs();
				}
				if(ApplicationParameters.getParameterValue("collect")){
					//check all the necessary parameter are set
					System.out.println("FILL");
					dbOp.CreateEvolutionTables();
				}
				if(ApplicationParameters.getParameterValue("statistics")){
					//check all the necessary parameter are set
					System.out.println("STATISTICS");
					StatisticsGenerator stat = new StatisticsGenerator();
					//execute all the statistics set in the configuration file
					stat.executeStatistics();
				}
			}
		}
		catch(SQLException sEx){
			System.out.println("An error is occured during the required operation:  " + sEx.getMessage());
			System.out.println("The operation will be aborted ");
			System.exit(1);
		}
	}
	
	
	/*private static boolean checkParametersRepository(){
		if(ApplicationParameters.getRepoType().toLowerCase().equals("svn")){
			//check svn parameters
			if(ApplicationParameters.getSvnurl().equals("")){
				System.out.println("SET THE SVN URL OF THE SCHEMA");
				return false;
			}
		}
		else if(ApplicationParameters.getRepoType().toLowerCase().equals("cvs")){
			//check cvs parameters
			if(ApplicationParameters.getCvsLocalPath().equals("") || ApplicationParameters.getCvsmodule().equals("") || ApplicationParameters.getCvsRoot().equals("")){
				System.out.println("SET THE CVS PARAMETERS");
				return false;
			}
		}
		return true;
	}*/
	
	/**
	 * TODO
	 */
	private static void overrideFromCommandLine(CAConfiguration conf){
		if(((String)conf.getParameterValue("repositoryType")).equals("svn")){
			if((String) conf.getParameterValue("svnurl")!=null)
				ApplicationParameters.setSvnurl((String) conf.getParameterValue("svnurl"));
			if((String) conf.getParameterValue("svnuser")!=null)
				ApplicationParameters.setSvnuser((String) conf.getParameterValue("svnuser"));
			if((String) conf.getParameterValue("svnpwd")!=null)
			ApplicationParameters.setSvnpwd((String) conf.getParameterValue("svnpwd"));  
		}
		else if (((String)conf.getParameterValue("repositoryType")).equals("cvs")){
			if((String) conf.getParameterValue("cvsRoot")!=null)
				ApplicationParameters.setCvsRoot((String) conf.getParameterValue("cvsRoot"));
			if((String) conf.getParameterValue("cvsLocalPath")!=null)
				ApplicationParameters.setCvsLocalPath((String) conf.getParameterValue("cvsLocalPath"));
			if((String) conf.getParameterValue("cvsModule")!=null)
				ApplicationParameters.setCvsmodule((String) conf.getParameterValue("cvsModule"));
			if((String) conf.getParameterValue("cvsSaveSchemaPath")!=null)
				ApplicationParameters.setCvsSaveSchemaPath((String) conf.getParameterValue("cvsSaveSchemaPath"));
		}
		if((String) conf.getParameterValue("pathtoschema")!=null)
			ApplicationParameters.setPathtoschema((String) conf.getParameterValue("pathtoschema"));
		if((String) conf.getParameterValue("drop")!=null)
			ApplicationParameters.addParameter("drop", Boolean.parseBoolean((String)conf.getParameterValue("drop")));
		if((String) conf.getParameterValue("download")!=null)
			ApplicationParameters.addParameter("download", Boolean.parseBoolean((String)conf.getParameterValue("download")));
		if((String) conf.getParameterValue("install")!=null)
			ApplicationParameters.addParameter("install", Boolean.parseBoolean((String)conf.getParameterValue("install")));
		if((String) conf.getParameterValue("collect")!=null)
			ApplicationParameters.addParameter("collect", Boolean.parseBoolean((String)conf.getParameterValue("collect")));
		if((String) conf.getParameterValue("global")!=null)
			ApplicationParameters.addParameter("global", Boolean.parseBoolean((String)conf.getParameterValue("global")));
		if((String) conf.getParameterValue("dbbasename")!=null)
			ApplicationParameters.setDbbasename((String) conf.getParameterValue("dbbasename"));
	}
	
	private static void setCollectionTableInfo(){
		CAConfiguration conf = null;

		CAConfigFileParser filepar = CAConfigurator.getConfigFileParserInstance();
		conf = filepar.parse(ApplicationParameters.getCollectionQuery());
		
		//iterator over the keyset, all the keys of the hashmap 
		Iterator it = conf.keys();
		while(it.hasNext()){
			//print all the keys of the statistics file
			String table = it.next().toString();
			ApplicationParameters.addCollectionTable(table, (String)conf.getParameterValue(table));
		}
	}
	

}
