package edu.ucla.cs.dbevolution;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.SQLException;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * The class read a (sql) file containg a sql script and return a Vector of Query extracted from the file 
 * @author fangar
 *
 */


public class SqlScript {

	public final static char QUERY_ENDS = ';';
	public final static String QUERY_COMMENT = "--";
	public final static String QUERY_COMMENT_2 = "#";
	public final static String INSERT_STMT = "INSERT INTO";

	//private Connection conn;
	private BufferedReader rdr;
	//private Statement statement;
	private Vector<Query> queries;

	private boolean commaToDelete = false;
	
	/**
	 * Constructor: takes a bufferedReader and a sql connection to create the SqlScript object.
	 * Note that construction does not automatically read the script.
	 * @param bufRdr BufferedReader to the script data.
	 * @param connection SQL Connection
	 * @throws SQLException
	 */
	
	public SqlScript(String fileName) {
	
		queries = new Vector<Query>();
		try{
			rdr = new BufferedReader(new FileReader(fileName));
		}
		catch(FileNotFoundException f){
			System.out.println(f.getMessage());
		}
	}
	

	/**
	 * Loads the Sql Script from the BufferedReader and parses it into a statement.
	 * @throws IOException
	 */
	public Vector<Query> loadScript() throws IOException/*, SQLException*/ {
		String line;
		StringBuffer query = new StringBuffer();
		boolean queryEnds = false;
		boolean bracketToClose = false;
		
		//boolean tempflag = false;

		while ((line = rdr.readLine()) != null) {

			if (indexOfComment(line)>=0) {
				line = line.substring(0,indexOfComment(line));
			}
			line=cleanExtraComment(line);
			line = line.trim();
			if (line.equals("") || line.equals(";")) 
				continue;
			
			//check the line is an INSERT statement and in case of multiple line statement continue until all the statement has been bypassed
			if(IsInsertStatement(line)){
				//if the line contains the QUERY_END, it means the INSERT stmt is single-line
				//otherwise, it looks for the end of the stmt and bypass all the line from the beginnig to the end
				//check if there is the QUERY_END into the parsed line
				while(!checkStatementEnds(line)){
					line=rdr.readLine();
				}
				//I've read all the line of the INSERT stmt, the last line contains the QUERY_END, bypass the last line
				continue;
			}
			
			//check if the line is the first line of a CREATE statement
			if(isCreateStatement(line)){
				query.append(line);		
				continue;
			}
			
			//check wich is the degree of the cleaning process
			//if the degree if zero, continue without cleaning the line
			//otherwise clean the line with the required level
			
			line = quoteReservedWords(line);
			
			if(ApplicationParameters.getCleaningType().equals("light")){
				line = cleaningLight(line);
			}
			else if (ApplicationParameters.getCleaningType().equals("full")){
				int index = cleaningFull(line);
				
				Pattern pLength = Pattern.compile("\\)",Pattern.CASE_INSENSITIVE);
				Matcher mLength = pLength.matcher(line);
				
				Pattern pOpenSquare = Pattern.compile("\\(",Pattern.CASE_INSENSITIVE);
				Matcher mOpenSquare = pOpenSquare.matcher(line);
				
				Pattern pClosedSquare = Pattern.compile("\\)",Pattern.CASE_INSENSITIVE);
				Matcher mClosedSquare = pClosedSquare.matcher(line);
				
				if(index!=0){
					/*if(mLength.find(0)){
						index = mLength.end();
						mLength.reset();
					}
					if(line.endsWith(",")){
						line =  line.substring(0,index);
						line += ",";
					}
					else{
						line =  line.substring(0,index);
					}*/
					
					if(mOpenSquare.find(0)){
						if(mClosedSquare.find(0)){
							index = mClosedSquare.end();
						}
						else{
							//search the closed square bracket in the next line
							bracketToClose = true;
							query.append(line);
							continue;
						}
					}
					if(line.endsWith(",")){
						line =  line.substring(0,index);
						line += ",";
					}
					else{
						line =  line.substring(0,index);
					}
					
				}
				else if(bracketToClose){
					if(mClosedSquare.find(0)){
						index = mClosedSquare.end();
						
						if(line.endsWith(",")){
							line =  line.substring(0,index);
							line += ",";
						}
						else{
							line =  line.substring(0,index);
						}
					}
					bracketToClose = false;
				}
				if(line.trim().equals(""))
					continue;
			}
			
			queryEnds = checkStatementEnds(line);
			query.append(line);			
			//System.out.println(line);
			if (queryEnds) {
				if(commaToDelete){
					int index = query.toString().lastIndexOf(",");
					//System.out.println("INDICE: " + index);
					queries.add(new Query(removeCharAt(query.toString(), index),""));
				}
				else{
					queries.add(new Query(query.toString(),""));
				}
				//queries.add(new Query(query.toString(),""));
				//System.out.println(removeCharAt(query.toString(), index));
				//lastHadComma = false;
				query.setLength(0);
				commaToDelete=false;
			}
			//if (line.equals("CREATE TABLE filearchive (")) {tempflag=true;}
			//if (line.equals("  INDEX rc_ip (rc_ip)")) {tempflag=false;}
			//if (tempflag==true) {
			//	System.out.println("line after :"+line);
			//	System.out.println("queryEnds:"+queryEnds);
			//}
		}
		/*for(Query xxx: queries){
			System.out.println(xxx.getQuerytext());
		}*/
		return queries;
	}

	
	private String quoteReservedWords(String line) {
		
		Pattern pRelease = Pattern.compile("^release\\s",Pattern.CASE_INSENSITIVE);
		Matcher mRelease = pRelease.matcher(line);
		
		Pattern pRelease2 = Pattern.compile("(release\\s){1}",Pattern.CASE_INSENSITIVE);
		Matcher mRelease2 = pRelease2.matcher(line);
		
		/*if(line.contains("release")){
			line = line.replaceAll("release", "`release`");
		}*/
		if(mRelease.find(0)){
			line = mRelease.replaceFirst("`release`");
		}
		return line;
	}


	private String removeCharAt(String s, int pos) {
		return s.substring(0,pos)+s.substring(pos+1);
	}
	
	/**
	 * Clean the line deleting only the blob/text default value +
	 * Clean the line with the zero  cleaning
	 * @param line
	 * @return
	 */
	private String cleaningLight(String line){
		line = clenBlobField(line);
		return line;
	}
	
	
	/**
	 * Clean the line from all the sql ddl except the data type +
	 * Clean the line with the zero cleaning + 
	 * Clean the line with the light cleaning
	 * @param line
	 * @return
	 */
	private int cleaningFull(String line){
		
		int lastIndex = 0;
		
		Vector<Matcher> matchers = new Vector<Matcher>();
		Vector<Pattern> patterns = new Vector<Pattern>();

		//check wich is the datatype of the field defined in the line, and clean all the characters after it
		//clean default value, null or not null etc....
		//drop the primary key
		//if the line doesn't define a datatype, clean the line

		Pattern pBit = Pattern.compile("(\\sbit){1}",Pattern.CASE_INSENSITIVE);
		Matcher mBit = pBit.matcher(line);
		matchers.add(mBit);
		
		Pattern pTinyInt = Pattern.compile("(\\stinyint){1}",Pattern.CASE_INSENSITIVE);
		Matcher mTinyInt = pTinyInt.matcher(line);
		matchers.add(mTinyInt);

		Pattern pSmallInt = Pattern.compile("(\\ssmallint){1}",Pattern.CASE_INSENSITIVE);
		Matcher mSmallInt = pSmallInt.matcher(line);
		matchers.add(mSmallInt);
		
		Pattern pMediumInt = Pattern.compile("(\\smediumint){1}",Pattern.CASE_INSENSITIVE);
		Matcher mMediumInt = pMediumInt.matcher(line);
		matchers.add(mMediumInt);
		
		Pattern pInt = Pattern.compile("(\\sint){1}",Pattern.CASE_INSENSITIVE);
		Matcher mInt = pInt.matcher(line);
		matchers.add(mInt);
		
		Pattern pInteger = Pattern.compile("(\\sinteger){1}",Pattern.CASE_INSENSITIVE);
		Matcher mInteger = pInteger.matcher(line);
		matchers.add(mInteger);
		
		Pattern pBigInt = Pattern.compile("(\\sbigint){1}",Pattern.CASE_INSENSITIVE);
		Matcher mBigInt = pBigInt.matcher(line);
		matchers.add(mBigInt);
		
		Pattern pReal = Pattern.compile("(\\sreal){1}",Pattern.CASE_INSENSITIVE);
		Matcher mReal = pReal.matcher(line);
		matchers.add(mReal);
		
		Pattern pDouble = Pattern.compile("(\\sdouble){1}",Pattern.CASE_INSENSITIVE);
		Matcher mDouble = pDouble.matcher(line);
		matchers.add(mDouble);
		
		Pattern pFloat = Pattern.compile("(\\sfloat){1}",Pattern.CASE_INSENSITIVE);
		Matcher mFloat = pFloat.matcher(line);
		matchers.add(mFloat);
		
		Pattern pDecimal = Pattern.compile("(\\sdecimal){1}",Pattern.CASE_INSENSITIVE);
		Matcher mDecimal = pDecimal.matcher(line);
		matchers.add(mDecimal);
		
		Pattern pNumeric = Pattern.compile("(\\snumeric){1}",Pattern.CASE_INSENSITIVE);
		Matcher mNumeric = pNumeric.matcher(line);
		matchers.add(mNumeric);
		
		Pattern pDate = Pattern.compile("(\\sdate){1}",Pattern.CASE_INSENSITIVE);
		Matcher mDate = pDate.matcher(line);
		matchers.add(mDate);
		
		Pattern pTime = Pattern.compile("(\\stime){1}",Pattern.CASE_INSENSITIVE);
		Matcher mTime = pTime.matcher(line);
		matchers.add(mTime);
		
		Pattern pTimestamp = Pattern.compile("(\\stimestamp){1}",Pattern.CASE_INSENSITIVE);
		Matcher mTimestamp = pTimestamp.matcher(line);
		matchers.add(mTimestamp);
		
		Pattern pDatetime = Pattern.compile("(\\sdatetime){1}",Pattern.CASE_INSENSITIVE);
		Matcher mDatetime = pDatetime.matcher(line);
		matchers.add(mDatetime);
		
		Pattern pYear = Pattern.compile("(\\syear){1}",Pattern.CASE_INSENSITIVE);
		Matcher mYear = pYear.matcher(line);
		matchers.add(mYear);
		
		Pattern pChar = Pattern.compile("(\\schar){1}",Pattern.CASE_INSENSITIVE);
		Matcher mChar = pChar.matcher(line);
		matchers.add(mChar);
		
		Pattern pVarchar = Pattern.compile("(\\svarchar){1}",Pattern.CASE_INSENSITIVE);
		Matcher mVarchar = pVarchar.matcher(line);
		matchers.add(mVarchar);
		
		Pattern pBinary = Pattern.compile("(\\sbinary){1}",Pattern.CASE_INSENSITIVE);
		Matcher mBinary = pBinary.matcher(line);
		matchers.add(mBinary);
		
		Pattern pVarBinary = Pattern.compile("(\\svarbinary){1}",Pattern.CASE_INSENSITIVE);
		Matcher mVarBinary = pVarBinary.matcher(line);
		matchers.add(mVarBinary);
		
		Pattern pTinyBlob = Pattern.compile("(\\stinyblob){1}",Pattern.CASE_INSENSITIVE);
		Matcher mTinyBlob = pTinyBlob.matcher(line);
		matchers.add(mTinyBlob);
		
		Pattern pBlob = Pattern.compile("(\\sblob){1}",Pattern.CASE_INSENSITIVE);
		Matcher mBlob = pBlob.matcher(line);
		matchers.add(mBlob);
		
		Pattern pMediumBlob = Pattern.compile("(\\smediumblob){1}",Pattern.CASE_INSENSITIVE);
		Matcher mMediumBlob = pMediumBlob.matcher(line);
		matchers.add(mMediumBlob);
		
		Pattern pLongBlob = Pattern.compile("(\\slongblob){1}",Pattern.CASE_INSENSITIVE);
		Matcher mLongBlob = pLongBlob.matcher(line);
		matchers.add(mLongBlob);
		
		Pattern pTinyText = Pattern.compile("(\\stinytext){1}",Pattern.CASE_INSENSITIVE);
		Matcher mTinyText = pTinyText.matcher(line);
		matchers.add(mTinyText);
		
		Pattern pText = Pattern.compile("(\\stext){1}",Pattern.CASE_INSENSITIVE);
		Matcher mText = pText.matcher(line);
		matchers.add(mText);
		
		Pattern pMediumText = Pattern.compile("(\\smediumtext){1}",Pattern.CASE_INSENSITIVE);
		Matcher mMediumText = pMediumText.matcher(line);
		matchers.add(mMediumText);
		
		Pattern pLongText = Pattern.compile("(\\slongtext){1}",Pattern.CASE_INSENSITIVE);
		Matcher mLongText = pLongText.matcher(line);
		matchers.add(mLongText);
		
		Pattern pEnum = Pattern.compile("(\\senum){1}",Pattern.CASE_INSENSITIVE);
		Matcher mEnum = pEnum.matcher(line);
		matchers.add(mEnum);
		
		Pattern pSet = Pattern.compile("(\\sset){1}",Pattern.CASE_INSENSITIVE);
		Matcher mSet = pSet.matcher(line);
		matchers.add(mSet);
		
		
		//FINISHED QUERY PATTERN
		Pattern pCharset = Pattern.compile("(\\scharset){1}",Pattern.CASE_INSENSITIVE);
		Matcher mCharset = pSet.matcher(line);

		
		patterns.add(pBit);
		patterns.add(pTinyInt);
		patterns.add(pSmallInt);
		patterns.add(pMediumInt);
		patterns.add(pInt);
		patterns.add(pInteger);
		patterns.add(pBigInt);
		patterns.add(pReal);
		patterns.add(pDouble);
		patterns.add(pFloat);
		patterns.add(pDecimal);
		patterns.add(pNumeric);
		patterns.add(pDate);
		patterns.add(pTime);
		patterns.add(pTimestamp);
		patterns.add(pDatetime);
		patterns.add(pYear);
		patterns.add(pChar);
		patterns.add(pVarchar);
		patterns.add(pBinary);
		patterns.add(pVarBinary);
		patterns.add(pTinyBlob);
		patterns.add(pBlob);
		patterns.add(pMediumBlob);
		patterns.add(pLongBlob);
		patterns.add(pTinyText);
		patterns.add(pText);
		patterns.add(pMediumText);
		patterns.add(pLongText);
		patterns.add(pEnum);
		patterns.add(pSet);
		
		//drop the primary key
		//drop the unique definition
		//drop any other statement
		/*Pattern pConstraint = Pattern.compile("(^constraint){1}",Pattern.CASE_INSENSITIVE);
		Matcher mConstraint = pConstraint.matcher(line);
		
		Pattern pIndex = Pattern.compile("(^index){1}",Pattern.CASE_INSENSITIVE);
		Matcher mIndex = pIndex.matcher(line);
		
		Pattern pKey = Pattern.compile("(^key){1}",Pattern.CASE_INSENSITIVE);
		Matcher mKey = pKey.matcher(line);
		
		Pattern pFullText = Pattern.compile("(^fulltext){1}",Pattern.CASE_INSENSITIVE);
		Matcher mFullText = pFullText.matcher(line);
		
		Pattern pSpatial = Pattern.compile("(^spatial){1}",Pattern.CASE_INSENSITIVE);
		Matcher mSpatial = pSpatial.matcher(line);
		
		Pattern pCheck = Pattern.compile("(^check){1}",Pattern.CASE_INSENSITIVE);
		Matcher mCheck = pCheck.matcher(line);
		
		Pattern pUnique = Pattern.compile("(^unique){1}",Pattern.CASE_INSENSITIVE);
		Matcher mUnique = pUnique.matcher(line);
		
		Pattern pPrimary = Pattern.compile("(^primary){1}",Pattern.CASE_INSENSITIVE);
		Matcher mPrimary = pPrimary.matcher(line);
		*/
		//the following are the mysql data type defined with the structure
		//data_type_name [(length)]
		//data_type_name (length)
		//data_type_name [(length,decimal)] or 
		//data_type_name [(length[,decimal])]
		//data_type_name (value1,value2,value3..)
		//if a length is defined keep it and delete all the remaing 
		//otherwise keep the datatype and clean everything else
		//Pattern pLength = Pattern.compile("\\)",Pattern.CASE_INSENSITIVE);
		//Matcher mLength = pLength.matcher(line);
		/*
		if(mBit.find(0)){
			lastIndex = mBit.end();
		}
		if(mTinyInt.find(0)){
			lastIndex = mTinyInt.end(); 
		}
		if (mSmallInt.find(0)){
			lastIndex = mSmallInt.end();
		}
		if(mBinary.find(0)){
			lastIndex = mBinary.end();
		}
		if(mMediumInt.find(0)){
			lastIndex = mMediumInt.end();	
		}
		if(mInt.find(0)){
			lastIndex = mInt.end();	
		}
		if(mInteger.find(0)){
			lastIndex = mInteger.end();	
		}
		if(mBigInt.find(0)){
			lastIndex = mBigInt.end();	
		}
		if(mChar.find(0)){
			lastIndex = mChar.end();	
		}
		if(mReal.find(0)){
			lastIndex = mReal.end();	
		}
		if(mFloat.find(0)){
			lastIndex = mFloat.end();	
		}
		if(mDecimal.find(0)){
			lastIndex = mDecimal.end();	
		}
		if(mNumeric.find(0)){
			lastIndex = mNumeric.end();	
		}
		if(mEnum.find(0)){
			lastIndex = mEnum.end();	
		}
		if(mSet.find(0)){
			lastIndex = mSet.end();	
		}
		if(mVarBinary.find(0)){
			lastIndex = mVarBinary.end();	
		}
		if(mVarchar.find(0)){
			lastIndex = mVarchar.end();	
		}
		if(mDouble.find(0)){
			lastIndex = mDouble.end();	
		}
		/*if(lastIndex!=0){
			line =  line.substring(0,lastIndex);
		}*/
		
		/*if(mBit.find() || mTinyInt.find() || mSmallInt.find() || mBinary.find() || mMediumInt.find() || mInt.find() || mInteger.find() || mBigInt.find() || mChar.find() 
				|| mReal.find() || mFloat.find() || mDecimal.find() || mNumeric.find() || mEnum.find() || mSet.find()|| mVarBinary.find() || mVarchar.find()
				|| mDouble.find()){
			Pattern pLength = Pattern.compile("(.)",Pattern.CASE_INSENSITIVE);
			Matcher mLength = pLength.matcher(line);
			if(mLength.find()){
				//lastIndex = mLength.end();
				return line.substring(0,line.indexOf(")")+1) + ",";
			}
			/*else{
				lastIndex = mDouble.end();	
			}
		}*/
		
		
		
		//line =  line.substring(0,line.indexOf(")")+1) + ",";
		//line =  line.substring(0,lastIndex) + ",";
		//else{
		/*
		//the following are the mysql data type defined with the structure
		//data_type_name 
		//keep the datatype and clean everything else
		if(mDate.find()){
			lastIndex = mDate.end();
		}
		if (mDatetime.find()){
			lastIndex = mDatetime.end();
		}
		if (mTime.find()){
			lastIndex = mTime.end();
		}
		if (mTimestamp.find()){
			lastIndex = mTimestamp.end();
		}
		if(mYear.find()){
			lastIndex = mYear.end();
		}
		if(mTinyBlob.find()){
			lastIndex = mTinyBlob.end();
		}
		if(mBlob.find()){
			lastIndex = mBlob.end();
		}
		if(mMediumBlob.find()){
			lastIndex = mMediumBlob.end();
		}
		if (mLongBlob.find()){
			lastIndex = mLongBlob.end();
		}
		 if (mTinyText.find()){
			lastIndex = mTinyText.end();
		}
		if(mText.find()){
			lastIndex = mText.end();
		}
		if(mMediumText.find()){
			lastIndex = mMediumText.end();
		}
		if(mLongText.find()){
			lastIndex = mLongText.end();
		}*/
		//}
		/*if(lastIndex!=0){
			//replace all the characters from the end of the datatype to the end
			//System.out.println("LAST INDEX: " + lastIndex);
			if(line.endsWith(",")){
				line =  line.substring(0,lastIndex);
				line += ",";
			}
			else{
				line =  line.substring(0,lastIndex);
			}
		}*/
		/*if(mConstraint.find() || mIndex.find() || mKey.find() || mFullText.find() || mSpatial.find() || mCheck.find() 
				|| mUnique.find() || mPrimary.find()){
			line = "";
			//the last ddl element had comma while it shouldn't
			// report an example to clarify the statement
			//	CREATE TABLE....
			//  	xxxx int(11),
			//  	xxxx2 tinyint(2),
			//  	PRIMARY KEY  (xxxx)
			//) TYPE=MyISAM;
			
			//the xxxx2 has a comma while, once deleted the primary key statement the comma must be deleted
			//use a flag to check whether one of the previous matching succeed
			//if yes, delete the last comma from the create table stmt
			//commaToDelete = true;
			lastIndex = 0;
		}*/
		//mLength.reset();
		//return line;
		
		/*for(Matcher matcher:matchers){
			if(matcher.find()){
				if(matcher.end()>lastIndex){
					lastIndex = matcher.end();
				}
			}
		}*/
		
		Matcher m; 
		
		m = pCharset.matcher(line);
		if(m.find()){
			return line.length();
		}
		
		for(Pattern pattern:patterns){
			m = pattern.matcher(line);
			if(m.find()){
				if(m.end()>lastIndex){
					lastIndex = m.end();
				}
			}
		}
		
		return lastIndex;
		
	}
		
				
	
	
	/**
	 * Check whether or not the line is a comment
	 * Check wheter or not the comment is inside the line
	 * @param line
	 * @return true if there is a comment inside the line (wheter the entire line is a comment or only a part of it), false otherwise
	 */
	private int indexOfComment(String line) {
		int indexComment = -1;
		if ((line != null) && (line.length() > 0)){
			if(line.indexOf(QUERY_COMMENT)>=0){
				//return (line.indexOf(QUERY_COMMENT));
				indexComment = line.indexOf(QUERY_COMMENT);
			}
			if (line.indexOf(QUERY_COMMENT_2)>=0){
				if(( line.indexOf(QUERY_COMMENT_2)< indexComment ) && (indexComment != -1)){
					indexComment = line.indexOf(QUERY_COMMENT_2);
				}
				else if (indexComment == -1){
					indexComment = line.indexOf(QUERY_COMMENT_2);
				}
			}
		}
		return indexComment;
	}
	
	/**
	 * The method check whether the line is an sql INSERT statement
	 * It delete also multiple line INSERT statement
	 * @param line
	 */
	private boolean IsInsertStatement(String line){
		
		Pattern patternInsert = Pattern.compile("(INSERT){1}",Pattern.CASE_INSENSITIVE);
        Matcher matcherInsert = patternInsert.matcher(line);
		if(matcherInsert.find()){
			//System.out.println("THE LINE IS AN INSERT STMT:" + line);
			return true;
		}
		return false;
	}
	
	
	/**
	 * The method check whether the line is an sql INSERT statement
	 * It delete also multiple line INSERT statement
	 * @param line
	 */
	private boolean isCreateStatement(String line){
		
		Pattern patternCreate = Pattern.compile("(CREATE TABLE){1}",Pattern.CASE_INSENSITIVE);
        Matcher matcherCreate = patternCreate.matcher(line);
		if(matcherCreate.find()){
			return true;
		}
		return false;
	}

	/**
	 * Trick to avoid a java error ( the connector doesn't permit a create statement with a default value for blob field, while the mysql command line does) 
	 * Clean the sql line from the initialization of a blob field
	 * From the sql manual: "BLOB and TEXT columns cannot have DEFAULT values."
	 * The wikimedia script set a default value (default '') from the blob and text field
	 * The function remove the initialization of the field  
	 * @param the query line
	 * @return the cleaned query line
	 */
	private String clenBlobField(String line){
		
		Pattern patternText = Pattern.compile("(text){1}",Pattern.CASE_INSENSITIVE);
		Pattern patternBlob = Pattern.compile("(blob){1}",Pattern.CASE_INSENSITIVE);
		Pattern patternDefault = Pattern.compile("(default ''){1}",Pattern.CASE_INSENSITIVE);

        Matcher matcherText = patternText.matcher(line);
        Matcher matcherBlob = patternBlob.matcher(line);
        Matcher matcherDefault = patternDefault.matcher(line);
        
        while (matcherText.find() || matcherBlob.find()) {
        	//System.out.println("THE LINE HAS A BLOB/TEXT: " + line);
            //if(line.indexOf(defRemove)!=-1){
        	if(matcherDefault.find()){
            	//System.out.println("THE BLOB/TEXT HAS DEFAULT...");
    			//line = line.replaceFirst(defRemove,"");
        		line = line.substring(0,matcherDefault.start()) + line.substring(matcherDefault.end(),line.length());
    			//System.out.println("BLOB REPLACED: " + line);
            }
        }
        return line;
	}
	
	
	/**
	 * Clean the query line from other unwanted characters like for istance /* or the closing comment 
	 * @param line
	 * @return
	 */
	private String cleanExtraComment(String line){
		if((line.indexOf("/*")!=-1)&&(line.indexOf("*/")!=-1)){
			String del = line.substring(line.indexOf("/*"),line.indexOf("*/")+2);
			line = line.replace(del, "");
		}
		return line; 
	}
	
	/**
	 * Check whether or not the query is finished
	 * @param line of the query
	 * @return true if the query if finished, false otherwise
	 */
	private boolean checkStatementEnds(String s) {
		//check if the line containsa semi-column and if it's the last char 
		//insert statement with semicolumn in the middle of the line would file without the second check
		return ((s.indexOf(QUERY_ENDS) != -1) && s.indexOf(QUERY_ENDS) == s.length()-1);
	}


}
