package edu.ucla.cs.dbevolution;

import java.io.File;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Vector;

import org.tmatesoft.svn.core.SVNDepth;
import org.tmatesoft.svn.core.SVNException;
import org.tmatesoft.svn.core.SVNLogEntry;
import org.tmatesoft.svn.core.SVNURL;
import org.tmatesoft.svn.core.auth.ISVNAuthenticationManager;
import org.tmatesoft.svn.core.internal.io.dav.DAVRepositoryFactory;
import org.tmatesoft.svn.core.internal.io.fs.FSRepositoryFactory;
import org.tmatesoft.svn.core.internal.io.svn.SVNRepositoryFactoryImpl;
import org.tmatesoft.svn.core.io.SVNRepository;
import org.tmatesoft.svn.core.io.SVNRepositoryFactory;
import org.tmatesoft.svn.core.wc.SVNRevision;
import org.tmatesoft.svn.core.wc.SVNUpdateClient;
import org.tmatesoft.svn.core.wc.SVNWCUtil;

/**
 * 
 * The class group all the operation that must be performed against the svn repository
 * - download of the revision numbers from the repository
 * - download of the single schema from the repository
 * @author fangar
 *
 */
public class SvnOperations implements Repository{


	private static final String PROTOCOL = "http"; 

	
	
	//implements the GetRevisions method for the cvs repository
	public Vector<DbVersion> GetRevisions(){
        
		Vector<DbVersion> revisions = new Vector<DbVersion>();
		String url = ApplicationParameters.getSvnurl();
		DbVersion version;
        
        long startRevision = 0;
        long endRevision = -1;//HEAD (the latest) revision
        /*
         * Initializes the library (it must be done before ever using the
         * library itself
         * The connection to the svn server in this case is done via the http protocol
         */
        setupLibrary();

        SVNRepository repository = null;
        
        try {
            /*
             * Creates an instance of SVNRepository to work with the repository.
             * All user's requests to the repository are relative to the
             * repository location used to create this SVNRepository.
             * SVNURL is a wrapper for URL strings that refer to repository locations.
             */
            repository = SVNRepositoryFactory.create(SVNURL.parseURIEncoded(url));
        } catch (SVNException svne) {
            /*
             * Perhaps a malformed URL is the cause of this exception.
             */
            System.err.println("error while creating an SVNRepository for the location '" + url + "': " + svne.getMessage());
            System.exit(1);
        }
        
        repository.setAuthenticationManager(this.GetAuthManager());
        
        /*
         * Gets the latest revision number of the repository
         */
        try {
            endRevision = repository.getLatestRevision();
        } catch (SVNException svne) {
            System.err.println("error while fetching the latest repository revision: " + svne.getMessage());
            System.exit(1);
        }

        Collection logEntries = null;
        try {
            /*
             * Collects SVNLogEntry objects for all revisions in the range
             * defined by its start and end points [startRevision, endRevision].
             * For each revision commit information is represented by
             * SVNLogEntry.
             * 
             * the 1st parameter (targetPaths - an array of path strings) is set
             * when restricting the [startRevision, endRevision] range to only
             * those revisions when the paths in targetPaths were changed.
             * 
             * the 2nd parameter if non-null - is a user's Collection that will
             * be filled up with found SVNLogEntry objects; it's just another
             * way to reach the scope.
             * 
             * startRevision, endRevision - to define a range of revisions you are
             * interested in; by default in this program - startRevision=0, endRevision=
             * the latest (HEAD) revision of the repository.
             * 
             * the 5th parameter - a boolean flag changedPath - if true then for
             * each revision a corresponding SVNLogEntry will contain a map of
             * all paths which were changed in that revision.
             * 
             * the 6th parameter - a boolean flag strictNode - if false and a
             * changed path is a copy (branch) of an existing one in the repository
             * then the history for its origin will be traversed; it means the 
             * history of changes of the target URL (and all that there's in that 
             * URL) will include the history of the origin path(s).
             * Otherwise if strictNode is true then the origin path history won't be
             * included.
             * 
             * The return value is a Collection filled up with SVNLogEntry Objects.
             */
            logEntries = repository.log(new String[] {""}, null, startRevision, endRevision, true, true);

        } catch (SVNException svne) {
            System.out.println("error while collecting log information for '" + url + "': " + svne.getMessage());
            System.exit(1);
        }
        for (Iterator entries = logEntries.iterator(); entries.hasNext();) {
            /*
             * gets a next SVNLogEntry
             */
            SVNLogEntry logEntry = (SVNLogEntry) entries.next();
            //System.out.println("---------------------------------------------");
            /*
             * gets the revision number
             */
            System.out.println("revision: " + logEntry.getRevision());
            System.out.println("date: " + logEntry.getDate());
            
            //create a DbVersion object to contains all the data extraced from the svn site
            version = new DbVersion();
            version.set_dbName(ApplicationParameters.getDbbasename());
            version.set_dbVersion(Long.toString(logEntry.getRevision()));
            //version.set_dbVersionName(cleanRevisionNumber(Long.toString(logEntry.getRevision())));
            version.set_dateFrom(logEntry.getDate());
            revisions.add(version);
        }
        for(int i=0;i<revisions.size()-1;i++){	
			((DbVersion)revisions.get(i)).set_dateTo(((DbVersion)revisions.get(i+1)).get_dateFrom());
			//System.out.println("SET DATE TO");
		}
		//set the Date_to field of the last element to TODAY otherwise, the method that insert the values in the db throws an exception
		((DbVersion)revisions.lastElement()).set_dateTo(new Date());
    
        return revisions;
    }

	
	//implements the DownloadRevision method for the cvs repository
	public void DownloadRevision(String revision){
        
		String sqlExt = ".sql";
		
        SVNRevision startingRevision = SVNRevision.create(Long.parseLong(revision));
    	SVNRevision endingRevision = SVNRevision.create(Long.parseLong(revision));
		try{
        	
			setupLibrary();
			SVNURL url = SVNURL.parseURIEncoded(ApplicationParameters.getSvnurl());
			//SVNRepository repository = SVNRepositoryFactory.create(url);
			// 1° possibile way to downaload the schema
			/*
            
			//repository.setAuthenticationManager(this.GetAuthManager());
			 
            SVNNodeKind nodeKind = repository.checkPath("", -1);
            if (nodeKind == SVNNodeKind.NONE) {
                SVNErrorMessage err = SVNErrorMessage.create(SVNErrorCode.UNKNOWN, "No entry at URL ''{0}''", url);
                throw new SVNException(err);
            }
           	repository.getFile("tables.sql", revision, null,new FileOutputStream("./schemata/schema_"+ revision + ".sql"));*/
            
            // 2° possibile way
            //both the methods works fine but are slower than the method used in the Esport class (the class is an example provided by the svnkit home site)
            //but that method can't be applied for one file (or i couldn't do it up to now...) 
            
            SVNUpdateClient updateClient = new SVNUpdateClient(this.GetAuthManager(), SVNWCUtil.createDefaultOptions(true));
			File destination = new File(ApplicationParameters.getPathtoschema() + /*ApplicationParameters.getDbbasename() +*/ revision + sqlExt);
			updateClient.doExport(url, destination, startingRevision, endingRevision, null, true, SVNDepth.EMPTY);
		} catch (SVNException svne) {
            System.out.println("error while downloading the schema" + revision + ": " + svne.getMessage());
            System.exit(1);
        }
	}
	
	
	
	private ISVNAuthenticationManager GetAuthManager(){
		
		String name = "anonymous";
        String password = "anonymous";
        
		/*
         * User's authentication information (name/password) is provided via  an 
         * ISVNAuthenticationManager  instance.  SVNWCUtil  creates  a   default 
         * authentication manager given user's name and password.
         * 
         * Default authentication manager first attempts to use provided user name 
         * and password and then falls back to the credentials stored in the 
         * default Subversion credentials storage that is located in Subversion 
         * configuration area. If you'd like to use provided user name and password 
         * only you may use BasicAuthenticationManager class instead of default 
         * authentication manager:
         * 
         *  authManager = new BasicAuthenticationsManager(userName, userPassword);
         *  
         * You may also skip this point - anonymous access will be used. 
         */
        
        name = (ApplicationParameters.getSvnuser()!=null) ? ApplicationParameters.getSvnuser() : name;
        password =(ApplicationParameters.getSvnpwd()!=null) ? ApplicationParameters.getSvnpwd() : password;
        
        ISVNAuthenticationManager authManager = SVNWCUtil.createDefaultAuthenticationManager(name, password);
        
        return authManager;

	}
	
	
    /*
     * Initializes the library to work with a repository via 
     * different protocols.
     */
    private static void setupLibrary() {
        
    	if(PROTOCOL.equals("http")){
    	    		DAVRepositoryFactory.setup();
    	}
    	else if (PROTOCOL.equals("svn")){
    		SVNRepositoryFactoryImpl.setup();
    	}
    	else if (PROTOCOL.equals("file")){
    		FSRepositoryFactory.setup();
    	}
    }
}

