package edu.ucla.cs.dbevolution;

import java.io.File;
import java.util.Vector;

import edu.ucla.cs.statistics.StatisticsParameters;


/**
 * Class of utility available to all the program's class
 * @author fangar
 *
 */
public class Utils {

	/**
	 * Clean the revision number from special characthers, to avoid error or conflict during the installation of the db
	 * For example if the revision number contains a period, mysql fails the installation or creation of the db
	 * Up to now the only charachter that created this sort of problem was the period
	 * @param revision
	 * @return
	 */
	public static String cleanRevisionNumber(String revision){
		//System.out.println("REVISION: "+ revision);
		String newRev = revision.replace(".", "_"); 
		//System.out.println("NEW REV: "+ newRev);
		return newRev;
	}
	
	
	
	/**
	 * Check if the directory exist, if not create it
	 * @param dir path to the filesystem directory
	 * @return true if the directory has been created
	 */
	public static boolean checkCreateDir(String dir){
	
		File fdir = new File(dir);
		if (!fdir.exists()) {
			try {
				System.out.println("Creating directory: " + dir);
				boolean dirCreated = fdir.mkdir();
				if (dirCreated) {
					System.out.println("Directory schema path created");
					return true;
				}
			} catch (SecurityException sEx) {
				System.out.println("The user doesn't have the privileges to create the directory schema");
				System.exit(1);
			}
		}
		return true;
	}
	
	
	/**
	 * TODO
	 * @return
	 */
	public static String getIntroPage(){
		return  StatisticsParameters.getStatPagePath() + ApplicationParameters.getDbbasename() + StatisticsParameters.getIntroPage();
	}
	
	
	/**
	 * Get the difference between the tables of two different revisions
	 * @param first the first revision to compare
	 * @param second the second revision to compare
	 * @return the number of tables different between the two revisions
	 */
	public static int getDifference(Vector<String> first, Vector<String> second){
		
		int countDiff=0;
		for(String table: first){
			if(!second.contains(table)){
				countDiff++;
			}
		}
		return countDiff;
	}
	
}
