package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Vector;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.Utils;

public class ColumnsAddedDeleted extends Statistic implements StatisticRisk{

	
	private HashMap<String, Integer> columnsEvolutionChanges;
	
	
	public ColumnsAddedDeleted() {
			
		//create the statistic
		//set the statistic parameters
		setTitle("Table Added and Deleted");
		setStatDescription("The statistic show the number of columns added and deleted per table every change of revision ;\n The statistic doesn't show " +
				"the detail of the transition between one revision and the other, while in this way it is possibile to better understand the change between " +
				"the revisions looking into the modifications occured to the database schema");
		setSqlStmt("The sql compare the name of the tables of every revision with the table name of the previuos and following revision");
		setRange("Added/Deleted");
		setDomain("Revisions");
		setStatName("statTablesAddedDeleted");
		setChartOrientation("v");
		setChartType("bar");
		setTitlePage("Statistic: Tables Added/Deleted");
		setStatPage(ApplicationParameters.getDbbasename() + getStatName() + ".html");
		setStatGroup("tables");
	}
	
	
	
	/**
	 * Create the dataset for the statistic
	 * @return the dataset
	 * @throws SQLException
	 */
	public void execute() throws SQLException{
		
		int tableChanges = 0; 
		
		//array to store the column's name
		Vector<String> columnsBase = new Vector<String>();
		Vector<String> columnsAc = new Vector<String>();
		
		HashMap<String, Integer> columnsEvolutionChanges = new HashMap<String, Integer>();
		
		String dbName = ApplicationParameters.getDbbasename()+"%";
		
		Vector<String> tables = getDb().getTables(dbName);
    	
		for(String tableName: tables){
			
			System.out.println("COLUMN EVOLUTION - PERFORMING TABLE: "+ tableName);
			
			//retrieve the name of the tables for the revision and revision +1
			Vector<String[]> columnsInTable = getDb().getColumnsInTable(dbName, tableName);
			
			String revisionBase =  ((String[]) columnsInTable.get(0))[0];
			String columnNameBase =  ((String[]) columnsInTable.get(0))[1];
			
			//add the first column's name to the array
			//use the actual column's array
			columnsAc.add(columnNameBase);
			
			for(int count=1;count<columnsInTable.size();count++){
				
				String revisionAc = ((String[]) columnsInTable.get(count))[0];
				String columnNameAc = ((String[]) columnsInTable.get(count))[1];
				
				if(revisionAc.equals(revisionBase)){
					//same revision, add the column's name to the array
					columnsAc.add(columnNameAc);
				}
				else{
					//different column's name,
					if(columnsBase.size()!=0){
						//it's not the first comparison, the base array has already been initialized
						//compare the two list of columns
						int deleted = Utils.getDifference(columnsBase, columnsAc);
						int added = Utils.getDifference(columnsAc, columnsBase);
						
						tableChanges += deleted;
						tableChanges += added;
					}
					columnsBase = (Vector<String>)columnsAc.clone();		
					columnsAc.clear();
					columnsAc.add(columnNameAc);
					
					revisionBase = revisionAc;
				}
			}
			
			//add the number of changes to the riskmapping table
			columnsEvolutionChanges.put(tableName, tableChanges);
			tableChanges = 0;
		}
		setColumnsEvolutionChanges(columnsEvolutionChanges);

	}
	
	
	/**
	 * Return the risk factor associated to the statistic, corresponding to tableName
	 * @param tableName
	 * @return
	 */
	public int getRiskMapping(String tableName) {
		try{
			return columnsEvolutionChanges.get(tableName);
		}
		catch(NullPointerException nEx){
			return 0;
		}
	}



	public void setColumnsEvolutionChanges(
			HashMap<String, Integer> columnsEvolutionChanges) {
		this.columnsEvolutionChanges = columnsEvolutionChanges;
	}
	
	
}
