package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.Vector;

import edu.ucla.cs.dbevolution.ApplicationParameters;

public class ColumnsLifetime extends StatisticLifetime {

	public ColumnsLifetime() {
	
		super();
		setStatName("statColumnsLifetime");
		setStatDescription("The statistics show the lifetimes of each column: the life of the columns is the number of revision where it is present; " +
				"it doesn't necessary mean the column was live for consecutive revisions, this is just a qualitative chart to represent the number of revision where column was present");
		setSqlStmt("SELECT COLUMN_NAME, COUNT(*) FROM " + ApplicationParameters.getHeaderEvolutionTables() + "COLUMNS WHERE TABLE_SCHEMA LIKE ? GROUP BY TABLE_NAME,COLUMN_NAME ORDER BY count(*) DESC");
		setTitlePage("Statistic: Columns Lifetime");
		setRange("Lifetime");
		setDomain("Columns");
		setChartType("bar");
		setTitle("Columns Lifetime");
		setImageAlt("columns lifetime");
		setChartOrientation("h");
		setStatGroup("columns");
		
	}
	
	/**
	 * Call the executeLifetime method of the superclass giving in input the sql statement to retrieve the lifetime of the tables
	 * and the list of parameters
	 * @throws SQLException 
	 */
	public void executeColumnsLifetime() throws SQLException{
		
		//sql statement for the max lifetime
		String sql = "SELECT max(count) FROM (SELECT COLUMN_NAME, COUNT(*) AS count FROM " + ApplicationParameters.getHeaderEvolutionTables() + "COLUMNS WHERE TABLE_SCHEMA LIKE ? GROUP BY TABLE_NAME,COLUMN_NAME) AS x";
		//params for max lifetime
		String[] params = new String[]{ApplicationParameters.getDbbasename()+"%"};
		
		//retrieve the dataset values
		Vector<StatisticValue> values = getDb().GetStatCount(ApplicationParameters.getDbbasename(), this.getSqlStmt());
		
		//call the super method
		executeLifetimeStatistic(values, sql, params);
	}
	
	
}
