package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Vector;

import org.jfree.data.category.DefaultCategoryDataset;

import edu.ucla.cs.dbevolution.ApplicationParameters;


/**
 * Create the statistic that inspect every single table, looking for a change in the key
 * The statistic analyze a table, checking if the keys are changed during the different revisions
 * @throws SQLException
 */
public class ForeignKeyChanges extends Statistic implements StatisticRisk{
	
	
	/**
	 * TODO
	 * Set the default filed for the statistic
	 */
	public ForeignKeyChanges(){
		
		setTitle("Foreign Keys Changes");
		setStatDescription("The statistic show the number of primary keys modified during the lifetime of a table\n The aggregation level is the table name. " +
				"This can be used as an index of database stability");
		setSqlStmt("The sql analyze the number of columns belonging to the key in each revision of the databas during the table lifetime");
		setRange("Foreign key Changes");
		setDomain("Tables");
		setStatName("statForeignKeysChanges");
		setChartOrientation("h");
		setChartType("bar");
		setTitlePage("Statistic: Foreign Keys Changes");
		setStatPage(ApplicationParameters.getDbbasename() + getStatName() + ".html");
		setStatGroup("constraints");
	}
	
	
	private HashMap<String, Integer> tablesPrimaryChanges;
	
	
	/**
	 * TODO
	 * @throws SQLException 
	 * 
	 */
	public void execute() throws SQLException{
		DefaultCategoryDataset ds = foreignKeyChangesDataset(); 
		if( (ds.getColumnCount()!=0) || (ds.getRowCount()!=0) ){
			setDs(ds);
			executeStatistic(this);
		}
		else{
			this.setImageAlt("No data available for this statistics");
		}
	}
	
	
	/**
	 * Set the dataset of the statistic
	 * @param ds the DefaultCategoryDataset
	 */
	public void setDataset(DefaultCategoryDataset ds){
		setDs(ds);
	}

	
	/**
	 * TODO
	 * @return
	 * @throws SQLException
	 */
	public DefaultCategoryDataset foreignKeyChangesDataset() throws SQLException{
		
		//the analysis is performed for single index and for single table,
		//but the result are aggregated over the table to make compact the structure of the chart
		
		//counter for the changes in the columns name for the index and the entire table
		int changesColNameIndex = 0;
		int changesColNameTable = 0;
		
		int changesColAddedIndex = 0;
		int changesColAddedTable = 0;
		
		int countTableResult = 0;
		
		String nextTable = "";
		String dbName = ApplicationParameters.getDbbasename() + "%";
		
		//risk mapping
		HashMap<String,Integer> tablesIndexChanges = new HashMap<String, Integer>();
		
		//create the dataset
		DefaultCategoryDataset ds = new DefaultCategoryDataset();
		Vector<String[]> tables = getDb().getTablesWithForeignKeys(dbName);
		
		
		//create the CSV file and add the header to the file
		createCSVFile(this);
		addValueToCsv(this, new String[]{"TableName","IndexChanges - Count","IndexChange - Name"});
		
		//retrieve the tables with indexes
		//for(int countTbl = 0;countTbl<tables.size()-1;countTbl++){
		
		for(String[] tableIndexes: tables){
			
			String tableName = tableIndexes[0];
			String indexName = tableIndexes[1];

			System.out.println("INDEX_CHANGES - PERFORMING TABLE: " + tableName + ", INDEX: " + indexName);
			
			//for each index in the table
			//retrieve the revision where the table was present
			//get the columns belonging to the index and check if there've been changes
			
			Vector<String[]> result = getDb().getColumnsInIndex(dbName, tableName, indexName);
			
			Vector<String> colIndexBase = new Vector<String>();
			Vector<String> colIndexAc = new Vector<String>();
			
			//retrieve the first database name
			String tableSchemaBase = ( (String[] ) result.get(0))[0];
			//retrieve the first column name
			String columnNameBase = ( (String[] ) result.get(0))[1];
			//add the column to the vector containing the column name belonging to the index
			colIndexAc.add(columnNameBase);
			
			
			for(int i=1;i<result.size(); i++){
				
				
				String tableSchemaAc = ( (String[]) result.get(i) )[0]; 
				String columnNameAc = ( (String[]) result.get(i) )[1];
				
				if(tableSchemaAc.equals(tableSchemaBase)){
					//same table_schema, different column_name
					colIndexAc.add(columnNameAc);
				}
				else{
					//different table, compare the columns name
					if(colIndexBase.size()!=0){
						//already some column added to the list of columns
						//compare the two list of columns
						if(colIndexBase.size() == colIndexAc.size()){
							//compare the columns name
							for(int j=0;j<colIndexBase.size();j++){
								if(!colIndexBase.contains( (String) colIndexAc.get(j) )){
									System.out.println("POSSIBLE NAME CHANGE");
									changesColNameIndex++;
								}
							}
						}
						else{
							System.out.println("DIFFERENCE IN SIZE");
							changesColAddedIndex++;
							//colInPrimaryBase = (Vector<String>)colInPrimaryAc.clone();
						}
					}
					else{
						//colInPrimaryBase = (Vector<String>)colInPrimaryAc.clone();	
					}
					//the column base vector is empty, is the first time performing the check
					colIndexBase = (Vector<String>)colIndexAc.clone();		
					
					colIndexAc.clear();
					colIndexAc.add(columnNameAc);
					tableSchemaBase = tableSchemaAc;
					
				}
				
			}
			//the analysis for the table is ended, add the index counter to the table counter
			changesColAddedTable += changesColAddedIndex;
			changesColNameTable += changesColNameIndex; 
			
			//reset the index counter
			changesColNameIndex=0;
			changesColAddedIndex=0;
			
			//check which table the performance was referring to
			//I don't know if there isn't any index left in the table, but i can check if the next vector of result
			//contains another tuple for the same table
			
			try{
				 nextTable = ( (String[])tables.get(countTableResult+1))[0];
			}
			catch (ArrayIndexOutOfBoundsException e) {
				//it's the last tuple in the array
				nextTable = tableName;
			}
			if(!nextTable.equals(tableName)){
				//the next table is different, add the result to dataset, csv and to he risk mapping
				
				//add values to the risk mapping
				//add the value to the mapping only if there's a column added in the primary key
				tablesIndexChanges.put(tableName, changesColAddedTable);

				//add the value to the csv
				addValueToCsv(this, new String[]{tableName,String.valueOf(changesColAddedTable), String.valueOf(changesColNameTable)});
				
				//add values to the dataset
				addValueToDataset(ds, changesColNameTable, "colName", tableName);
				addValueToDataset(ds, changesColAddedTable, "colAdded", tableName);

				//reset the counters
				changesColAddedIndex=0;
				changesColAddedTable=0;
				
				changesColNameTable=0;
				changesColNameIndex=0;
			}
			/*else{
				//add the counter of the index to the counter of the table
				changesColAddedTable += changesColAddedIndex;
				changesColNameTable += changesColNameIndex; 
			}*/
			countTableResult++;
			
		}
		//set the risk mapping to the local one
		setTablesPrimaryChanges(tablesIndexChanges);
		
		return ds;
	}


	/**
	 * TODO
	 * @param tableName
	 * @return
	 */
	public int getRiskMapping(String tableName) {
		try{
			return tablesPrimaryChanges.get(tableName);
		}
		catch(NullPointerException nEx){
			return 0;
		}
	}



	public void setTablesPrimaryChanges(
			HashMap<String, Integer> tablesPrimaryChanges) {
		this.tablesPrimaryChanges = tablesPrimaryChanges;
	}
	
}
