package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Vector;

import org.jfree.data.category.DefaultCategoryDataset;

import edu.ucla.cs.dbevolution.ApplicationParameters;


/**
 * Create the statistic that inspect every single table, looking for a change in the key
 * The statistic analyze a table, checking if the keys are changed during the different revisions
 * @throws SQLException
 */
public class PrimaryKeyChanges extends Statistic implements StatisticRisk{
	
	
	/**
	 * TODO
	 * Set the default filed for the statistic
	 */
	public PrimaryKeyChanges(){
		
		setTitle("Primary Keys Changes");
		setStatDescription("The statistic show the number of primary keys modified during the lifetime of a table\n The aggregation level is the table name. " +
				"This can be used as an index of database stability");
		setSqlStmt("The sql analyze the number of columns belonging to the key in each revision of the databas during the table lifetime");
		setRange("Primary key Changes");
		setDomain("Tables");
		setStatName("statPrimaryKeysChanges");
		setChartOrientation("h");
		setChartType("bar");
		setTitlePage("Statistic: Primary Keys Changes");
		setStatPage(ApplicationParameters.getDbbasename() + getStatName() + ".html");
		setStatGroup("constraints");
	}
	
	
	private HashMap<String, Integer> tablesPrimaryChanges;
	
	
	/**
	 * TODO
	 * @throws SQLException 
	 * 
	 */
	public void execute() throws SQLException{
		setDs(primaryKeyChangesDataset());
		executeStatistic(this);
	}
	
	
	/**
	 * Set the dataset of the statistic
	 * @param ds the DefaultCategoryDataset
	 */
	public void setDataset(DefaultCategoryDataset ds){
		setDs(ds);
	}

	
	/**
	 * TODO
	 * @return
	 * @throws SQLException
	 */
	public DefaultCategoryDataset primaryKeyChangesDataset() throws SQLException{
		
		//counter for the other method analysis
		int changesInColName = 0;
		int changesColAdded = 0;
		
		String db = ApplicationParameters.getDbbasename() + "%";
		
		//risk mapping
		HashMap<String,Integer> tablesPrimaryChanges = new HashMap<String, Integer>();
		
		//create the dataset
		DefaultCategoryDataset ds = new DefaultCategoryDataset();
		Vector<String> tables = getDb().getTablesWithKeys(db);
		
		
		//create the CSV file and add the header to the file
		createCSVFile(this);
		addValueToCsv(this, new String[]{"TableName","Changes Columns", "Changes Col Name"});
		
		for(int countTbl = 0;countTbl<tables.size()-1;countTbl++){
			
			String tableName = tables.get(countTbl);
			System.out.println("PERFORMING TABLE: " + tableName);

			
			/* BEGIN NEW METHOD		*/
			
			Vector<String[]> result = getDb().getColumnsInPrimary(tableName, db);
			
			Vector<String> colInPrimaryBase = new Vector<String>();
			Vector<String> colInPrimaryAc = new Vector<String>();
			
			//retrieve the first database name
			String dbBase = ( (String[] ) result.get(0))[0];
			//retrieve the first column name
			String columnBase = ( (String[] ) result.get(0))[1];
			//add the column to the vector containing the column name belonging to the primary key
			colInPrimaryAc.add(columnBase);
			
			
			for(int i=1;i<result.size(); i++){
				
				
				String dbAc = ( (String[]) result.get(i) )[0]; 
				String colAc = ( (String[]) result.get(i) )[1];
				
				if(dbAc.equals(dbBase)){
					//same table_schema, different column_name
					colInPrimaryAc.add(colAc);
				}
				else{
					//different table, compare the columns name
					if(colInPrimaryBase.size()!=0){
						//already some column added to the list of columns
						//compare the two list of columns
						if(colInPrimaryBase.size() == colInPrimaryAc.size()){
							//compare the columns name
							for(int j=0;j<colInPrimaryBase.size();j++){
								if(!colInPrimaryBase.contains( (String) colInPrimaryAc.get(j) )){
									System.out.println("POSSIBLE NAME CHANGE");
									changesInColName++;
								}
							}
						}
						else{
							System.out.println("DIFFERENCE IN SIZE");
							changesColAdded++;
							//colInPrimaryBase = (Vector<String>)colInPrimaryAc.clone();
						}
					}
					else{
						//colInPrimaryBase = (Vector<String>)colInPrimaryAc.clone();	
					}
					//the column base vector is empty, is the first time performing the check
					colInPrimaryBase = (Vector<String>)colInPrimaryAc.clone();		
					
					colInPrimaryAc.clear();
					colInPrimaryAc.add(colAc);
					dbBase = dbAc;
					
				}
				
			}
			//the analysis for the table is ended, set all the values
			
			//add values to the risk mapping
			//add the value to the mapping only if there's a column added in the primary key
			tablesPrimaryChanges.put(tableName, changesColAdded);
			
			//add values to the dataset
			addValueToDataset(ds, changesInColName, "colName", tableName);
			addValueToDataset(ds, changesColAdded, "colAdded", tableName);
			
			//add the value to the csv
			addValueToCsv(this, new String[]{tableName,String.valueOf(changesColAdded),String.valueOf(changesInColName)});
			
			changesColAdded=0;
			changesInColName=0;
			/*	END NEW METHOD		*/
		

		}
		//set the risk mapping to the local one
		setTablesPrimaryChanges(tablesPrimaryChanges);
		
		return ds;
	}


	/**
	 * TODO
	 * @param tableName
	 * @return
	 */
	public int getRiskMapping(String tableName) {
		try{
			return tablesPrimaryChanges.get(tableName);
		}
		catch(NullPointerException nEx){
			return 0;
		}
	}


	public void setTablesPrimaryChanges(
			HashMap<String, Integer> tablesPrimaryChanges) {
		this.tablesPrimaryChanges = tablesPrimaryChanges;
	}
	
	
}
