package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.Vector;

import edu.ucla.cs.dbevolution.ApplicationParameters;

/**
 * Create the statistic "Revisions Lifetime" highlighting the lifetime of each single revision
 * @author fangar
 */
public class RevisionsLifetime extends StatisticLifetime{

	/**
	 * Set the default filed for the statistic
	 */
	public RevisionsLifetime(){

		setTitle("Revision Lifetime");
		setStatDescription("The statistics show the lifetime, in days, of each revision");
		setSqlStmt("SELECT schema_unique_name, DATEDIFF(tend,tstart) FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_unique_name like ? AND tend < (SELECT MAX(tend) FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_unique_name like ?) ORDER BY tstart");
		setRange("Days");
		setDomain("Revisions");
		setStatName("statRevisionsLifetime");
		setChartOrientation("h");
		
		setChartType("bar");
		setTitlePage("Statistic: Revisions Lifetime");
		setStatGroup("revisions");
	}
	


	/**
	 * Execute the statistic
	 * @throws SQLException
	 */
	public void executeRevisionLifetime() throws SQLException{
		//sql statement for the max lifetime
		String sql = "SELECT MAX(date_diff) FROM (SELECT schema_unique_name, DATEDIFF(tend,tstart) AS date_diff FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_unique_name like ? " +
				"AND tend < (SELECT MAX(tend) FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_unique_name like ?) ORDER BY tstart) AS x";
		//params for max lifetime
		String[] params = new String[]{ApplicationParameters.getDbbasename()+"%", ApplicationParameters.getDbbasename()+"%"};

		//retrieve the dataset values
		Vector<StatisticValue> values = getDb().getRevisionLifetime(this);
		
		//call the super method
		executeLifetimeStatistic(values, sql, params);
	}

	
	
}
