package edu.ucla.cs.statistics;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import edu.ucla.cs.dbevolution.ApplicationParameters;

public class RiskAnalysis extends Statistic{

	
	
	//list of statistics containing the risk mapping
	private ArrayList<Statistic> statsRisk;
	private ArrayList<RiskResult> results;
	
	private double databaseRisk;
	
	private DatabaseSpecs databaseSpecs;
	
	/**
	 * @throws SQLException
	 */
	public /*ArrayList<RiskResult>*/void executeRiskAnalysis() throws SQLException{
		
		double globalRiskFactor=0.0;
		databaseSpecs = new DatabaseSpecs();
		
		//retrieve the name of all the tables present in the database
		Vector<String> tables = getDb().getTables(ApplicationParameters.getDbbasename()+"%");
		
		//list of risk result computed during the analysis
		ArrayList<RiskResult> results = null;
		
		
		if(tables!=null){
						
			results = new ArrayList<RiskResult>();
			
			for(String table:tables){
				
				RiskResult result = new RiskResult(table);
				
				Hashtable<String, Hashtable<String,String>> changes = getTableColumnsEvolutionDataType(table);
				result.setTypeChange(changes);
				
				//fill the remaining risk factor previously computed
				fillRiskFactors(result);
				result.computeRiskFactor();
				
				//add the table's risk factor to the global risk factor
				globalRiskFactor+=result.getTableRisk();
				results.add(result);
			}
			globalRiskFactor = globalRiskFactor/results.size();
			//this.databaseRisk = globalRiskFactor;
			
			//compute all the data related to the database history
			this.databaseSpecs.setDatabaseEvolutionRate(globalRiskFactor);
			this.databaseSpecs.setFirstRelease(getDb().getDatabaseFirstRelease());
			this.databaseSpecs.setLastRelease(getDb().getDatabaseLastRelease());
			this.databaseSpecs.setInitialSize(getDb().getDatabaseInitialSize());
			this.databaseSpecs.setEndSize(getDb().getDatabaseEndSize());
		}
		this.setResults(results);
	}
	
	
	public String getDatabaseRiskFactor() {
		
		//this.computeRiskFactor();
		//DecimalFormat format = new DecimalFormat("##.##");
		DecimalFormat format = new DecimalFormat("%");
		format.setPositivePrefix("");
		format.setPositiveSuffix("%");
		return format.format(databaseRisk);
	}
	
	
	/**
	 * Fill the risk result object with the risk factors computed during the previous statistics generation
	 * @param result
	 */
	private void fillRiskFactors(RiskResult result) {
		
		for(Statistic stat : statsRisk){
			if(stat instanceof PrimaryKeyChanges){
				result.setPrimaryKeyChanges(((PrimaryKeyChanges) stat).getRiskMapping(result.getTable()));
			}
			else if(stat instanceof ForeignKeyChanges){
				result.setForeignKeyChanges(((ForeignKeyChanges)stat).getRiskMapping(result.getTable()));
			}
			else if(stat instanceof RolledBackTables){
				result.setRollbackCount(((RolledBackTables)stat).getRiskMapping(result.getTable()));
			}
			else if(stat instanceof IndexChanges){
				result.setIndexChanges(((IndexChanges)stat).getRiskMapping(result.getTable()));
			}
			else if(stat instanceof ColumnsAddedDeleted){
				result.setColumnsEvolution(( (ColumnsAddedDeleted) stat).getRiskMapping(result.getTable()));
			}
		}
		
	}


	/**
	 * @param table
	 * @return
	 */
	private int getTablesRolledBackTest(String table){
		
		ArrayList<Savepoint> points = null;
		Savepoint sPrev = null;
		int rollbackCount = 0;

		try {
				System.out.println("PERFORMING TABLE: " + table);

				points = new ArrayList<Savepoint>();
				sPrev = null;

				
				//***************************************************/
				//			ALTERNATIVA CONNESSIONE MULTIPLA DB
				//***************************************************/
				
				/*for(String revision: revisions){
					
					//retrieve the columns names and the number of the columns
					Vector<String> columnsName = getDb().getColumnsName(revision, table);
					//create a savepoint to contain the columns name and number of columns
					Savepoint s = new Savepoint();
					s.setColumnsCount(columnsName.size());
					s.setColumnsName(columnsName);
					
					//compare the savepoint with the previous one
					//if different, add the savepoint, 
					//set the savepoint prev
					if(sPrev!=null){
						if(sPrev.getColumnsCount()!=s.getColumnsCount()){		//also to check the columns name
							//System.out.println("COL DIFFERENT: " + sPrev.getColumnsCount() + " WITH: " +s.getColumnsCount() );
							sPrev = s;
							points.add(s);
						}
					}
					else{
						sPrev = s;
					}
				}*/
				
				
				String sql = "SELECT table_schema,column_name FROM " + ApplicationParameters.getHeaderEvolutionTables() + "COLUMNS c, " + ApplicationParameters.getEvolutionTable() + " d WHERE c.table_schema = d.schema_unique_name " +
						"AND table_schema like ? AND table_name = ? ORDER BY d.tstart, table_schema, column_name";
				String[] params = new String[]{ApplicationParameters.getDbbasename() + "%", table};

				Vector<String[]> result = getDb().executeStatement(sql, params);
				
				if(result!=null){
				
					Vector<String> columnsName = new Vector<String>();
					
	
					String db = ( (String[]) result.get(0) )[0];
					String col = ( (String[]) result.get(0) )[1];
					
					columnsName.add(col);
					
					for(int i=1;i<result.size(); i++){
						
						String dbAc = ( (String[]) result.get(i) )[0]; 
						String colAc = ( (String[]) result.get(i) )[1];
						
						if(dbAc.equals(db)){
							columnsName.add(colAc);
						}
						else{
							Savepoint s = new Savepoint();
							s.setColumnsCount(columnsName.size());
							s.setColumnsName(columnsName);
							//points.add(s);
							
							if(sPrev!=null){
								if(sPrev.getColumnsCount()!=s.getColumnsCount()){
									sPrev = s;
									points.add(s);
								}
							}
							else{
								sPrev = s;
							}
							columnsName = new Vector<String>();
							columnsName.add(colAc);
							
							db = dbAc;
						}
					}
				}
				
				//search for a rolled back situation
				rollbackCount += checkTableRolledBack(points);
			}catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return rollbackCount;
	}
	
	
	


	/**
	 * @param points
	 */
	private int checkTableRolledBack(ArrayList<Savepoint> points) {
		
		int rollbackCount = 0;
		for(int countPoints = points.size()-1;countPoints>=0;countPoints--){
			Savepoint s = (Savepoint)points.get(countPoints);
			//compare savepoint with previous ones looking for rolledback situation
			for(int countPointsPrev =  0 + countPoints-1;countPointsPrev>=0;countPointsPrev--){
				if(s.compareSavepoints(s, points.get(countPointsPrev))){
					rollbackCount++;
				}
			}
		}
		return rollbackCount;
		
	}
	
	
	

	
	/**
	 * Compute the data type evolution for each table
	 * @param table
	 * @return
	 * @throws SQLException
	 */
	//public Hashtable<String, String> getTableColumnsEvolutionDataType(String table) throws SQLException{
	public Hashtable<String, Hashtable<String, String>> getTableColumnsEvolutionDataType(String table) throws SQLException{

		String sql = "SELECT table_schema,column_name, data_type FROM " + ApplicationParameters.getHeaderEvolutionTables() + 
		"COLUMNS c, " + ApplicationParameters.getEvolutionTable() + " d WHERE c.table_schema = d.schema_unique_name AND table_schema like ? AND table_name like ? GROUP BY column_name, table_schema" +
				" ORDER BY c.column_name ,d.tstart";
		String[] params = new String[]{ApplicationParameters.getDbbasename() + "%", table};

		boolean dataTypeChanges = false;
		//to store all the tables with a column that changed data type
		Hashtable<String, String> typeChanges = new Hashtable<String, String>();
		Hashtable<String, Hashtable<String, String>> typeChanges2 = new Hashtable<String, Hashtable<String,String>>();

		Vector<String[]> result = getDb().executeStatement(sql, params);

		if(result!=null){

			//DEBUG	String db = ( (String[]) result.get(0) )[0];	//only to compare in the debug
			String col = ( (String[]) result.get(0) )[1];
			String type = ( (String[]) result.get(0) )[2];

			for(int i=1;i<result.size(); i++){

				//DEBUG	String dbAc = ( (String[]) result.get(i) )[0]; 
				String colAc = ( (String[]) result.get(i) )[1];
				String typeAc = ( (String[]) result.get(i) )[2];

				/*if(colAc.equals(col) && typeAc.equals(type)){
				 * 
				 * 	not interested if the column name and table name are the same....
					//both the column name and the column datatype are equals, we're referring to the same column
					//of the same table, with the same datatyep
				}*/
				/*else */if(colAc.equals(col) && !typeAc.equals(type)){

					//the same column but with a different datatype
					dataTypeChanges = true;
					
					Hashtable<String, String> check = typeChanges2.get(col);
					if(check!=null){
						//a mapping already exist, check if the columns data type are the same
						String value = check.get(type);
						if(!value.equals(typeAc)){
							//the two data type aren't the same, i can add another column data type change
							Hashtable<String, String> toAdd = new Hashtable<String, String>();
							toAdd.put(type, typeAc);
							typeChanges2.put(col, toAdd);
						}
					}
					else{
						//no mapping exist, put a new mapping inside
						Hashtable<String, String> toAdd = new Hashtable<String, String>();
						toAdd.put(type, typeAc);
						typeChanges2.put(col, toAdd);
					}
				
					
				}
				else if(!colAc.equals(col)){
					//both the column and the datatype are different, we're changed the column.
					
					//before changing the column, check if a data type change occured, and save the column
					//in the hash table
					if(dataTypeChanges){
						typeChanges.put(col, type);
						dataTypeChanges = false;
					}
					
					//change the column
					col = colAc;
					type = typeAc;
				}
			}
		}
		//return typeChanges;
		return typeChanges2;
	}

	
	public ArrayList<Statistic> getStatsRisk() {
		return statsRisk;
	}


	public void setStatsRisk(ArrayList<Statistic> statsRisk) {
		this.statsRisk = statsRisk;
	}


	public void setDatabaseRisk(double databaseRisk) {
		this.databaseRisk = databaseRisk;
	}


	public double getDatabaseRisk() {
		return databaseRisk;
	}


	public void setResults(ArrayList<RiskResult> results) {
		this.results = results;
	}


	public ArrayList<RiskResult> getResults() {
		return results;
	}


	public void setDatabaseSpecs(DatabaseSpecs databaseSpecs) {
		this.databaseSpecs = databaseSpecs;
	}


	public DatabaseSpecs getDatabaseSpecs() {
		return databaseSpecs;
	}



	
	public class DatabaseSpecs {
		
		
		private String firstRelease;
		private String lastRelease;
		private int initialSize;
		private int endSize;
		private double databaseEvolutionRate;
		
		
		public String getFirstRelease() {
			return firstRelease;
		}
		public void setFirstRelease(String firstRelease) {
			this.firstRelease = firstRelease;
		}
		public String getLastRelease() {
			return lastRelease;
		}
		public void setLastRelease(String lastRelease) {
			this.lastRelease = lastRelease;
		}
		public int getInitialSize() {
			return initialSize;
		}
		public void setInitialSize(int initialSize) {
			this.initialSize = initialSize;
		}
		public int getEndSize() {
			return endSize;
		}
		public void setEndSize(int endSize) {
			this.endSize = endSize;
		}
		public double getDatabaseEvolutionRate() {
			return databaseEvolutionRate;
		}
		public void setDatabaseEvolutionRate(double databaseEvolutionRate) {
			this.databaseEvolutionRate = databaseEvolutionRate;
		}
		
		
		/**
		 * TODO
		 * @return
		 */
		public String getDatabaseEvolutionFactor() {
			
			//this.computeRiskFactor();
			//DecimalFormat format = new DecimalFormat("##.##");
			DecimalFormat format = new DecimalFormat("%");
			format.setPositivePrefix("");
			format.setPositiveSuffix("%");
			return format.format(getDatabaseEvolutionRate());
		}
		
		
		/**
		 * TODO..
		 * @return
		 */
		public String getBgPosition(){

			//round the decimal value with two decimal digit
			BigDecimal d = new BigDecimal(getDatabaseEvolutionRate());
			d.setScale(2,BigDecimal.ROUND_HALF_UP);
			double risk = d.doubleValue();
			
			//image width = 500;
			double shift = (1 - risk) * 100;
			
			return "" + String.valueOf(shift) + "%;";
		}
		
		

	}
	
	
	
	
	
	
	
	
	
}
