package edu.ucla.cs.statistics;

import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 * Parser for the statistic.xml file
 * The statistic file contains all the data to define a new statistic with the form (filed_name, count)
 * The parser read every statistic tag into the file and create a corresponding Statistic object
 * @author fangar
 *
 */
public class StatisticFileParser {

	/**
	 * Read the xml configuration file with the list of statistics to perform
	 * @param fileName 
	 * @return a list of statistics to execute
	 */
	public ArrayList<Statistic> parse(String fileName) {

		ArrayList<Statistic> stats = new ArrayList<Statistic>();

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		//String parameterName;
		//Object parameterValue;

		try {
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(fileName);
			Element root = doc.getDocumentElement();
			NodeList rootNodeList = root.getChildNodes();
			for (int i=0; i<rootNodeList.getLength();i++) {
				if (rootNodeList.item(i).getNodeName().compareTo("statistic")==0){

					//create a statistic to hold the configuration read from the xml file
					Statistic stat = new Statistic();
					String value;
					//retrieve all the parameters of the statistic
					NodeList statisticChild = rootNodeList.item(i).getChildNodes();

					//iterate over all the child of the statistic node
					for(int j=0;j<statisticChild.getLength();j++){
						Node child = statisticChild.item(j);
						if(child.getNodeType() == Node.ELEMENT_NODE){
							try{
								if(child.getNodeName().compareTo("stat-name")==0){
									value =child.getFirstChild().getNodeValue(); 
									stat.setStatName(value);
								}
								if(child.getNodeName().compareTo("stat-description")==0){
									value =child.getFirstChild().getNodeValue(); 
									stat.setStatDescription(value);
								}
								if(child.getNodeName().compareTo("stat-sql")==0){
									value =child.getFirstChild().getNodeValue(); 
									stat.setSqlStmt(value);
								}
								if(child.getNodeName().compareTo("stat-title-page")==0){
									value =child.getFirstChild().getNodeValue(); 
									stat.setTitlePage(value);
								}
								if(child.getNodeName().compareTo("stat-chart-range")==0){
									value =child.getFirstChild().getNodeValue(); 
									stat.setRange(value);
								}
								if(child.getNodeName().compareTo("stat-chart-domain")==0){
									value =child.getFirstChild().getNodeValue(); 
									stat.setDomain(value);
								}
								if(child.getNodeName().compareTo("stat-chart-type")==0){
									value =child.getFirstChild().getNodeValue(); 
									//check the chart type is among the available type.....
									//TODO
									stat.setChartType(value);
								}
								if(child.getNodeName().compareTo("stat-chart-title")==0){
									value =child.getFirstChild().getNodeValue(); 
									stat.setTitle(value);
								}
								if(child.getNodeName().compareTo("stat-chart-alt")==0){
									value =child.getFirstChild().getNodeValue(); 
									stat.setImageAlt(value);
								}
								if(child.getNodeName().compareTo("stat-chart-orientation")==0){
									value =child.getFirstChild().getNodeValue(); 
									//check the chart type is among the available type.....
									//TODO
									stat.setChartOrientation(value);
								}
								if(child.getNodeName().compareTo("stat-group")==0){
									value =child.getFirstChild().getNodeValue(); 
									//check the chart type is among the available type.....
									//TODO
									stat.setStatGroup(value);
								}
							}
							catch(NullPointerException nEx){
								System.out.println("Statistics file bad formed: the tag " + child.getNodeName() + " is missing the value");
								System.exit(1);
							}
						}
					}
					stats.add(stat);
				}
				//built in statistics
				else if (rootNodeList.item(i).getNodeName().compareTo("statistics_built_in")==0){

					String statName = "";
					boolean statValue = false;
					
					//retrieve all the built-in statistics
					//the built-in statistic are <statistic> tag
					//each tag contains the name of the statistic and a boolean value indicating whether or not the statistic must be
					//executed
					NodeList statisticChild = rootNodeList.item(i).getChildNodes();
					for(int k=0;k<statisticChild.getLength();k++){
						//get the child
						//the child is a single built-in statistic
						Node child = statisticChild.item(k);
						if(child.getNodeName().equals("statistic")){
							//get the built-in statistic parameters
							NodeList builtInChildren = child.getChildNodes();
							try{
								for(int m=0;m<builtInChildren.getLength();m++){
									if(builtInChildren.item(m).getNodeName().equals("statistic_name")){
										statName = builtInChildren.item(m).getFirstChild().getNodeValue();
									}
									if(builtInChildren.item(m).getNodeName().equals("statistic_exe")){
										statValue = Boolean.parseBoolean(builtInChildren.item(m).getFirstChild().getNodeValue());
									}
									if((statValue)&&(!statName.equals(""))){
										//add the statistic (statName) to the list of statistics to execute
										StatisticsParameters.addStat(statName);
										statValue = false;
										statName = "";
									}
								}
							}
							catch(NullPointerException nEx){
								//access to null xml value
								System.out.println("One or more tag in the statistic configuration file is missing;\n check the file and fill the missing parameter");
								System.exit(1);
							}
						}
					}
				}
			}
		}
		catch (ParserConfigurationException pce) {
			pce.printStackTrace(System.err);
			System.exit(1);
		}
		catch (IOException ioe) {
			ioe.printStackTrace(System.err);
			System.exit(1);
		}
		catch (SAXException saxe) {
			saxe.printStackTrace(System.err);
			System.exit(1);
		}
		return stats;
	}

}
