package edu.ucla.cs.statistics;

import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Parser for the groups.xml file
 * The groups file contains all the data to define a new groups of statistic
 * The parser read every group tag into the file and create a corresponding StatisticGroup object
 * @author fangar
 *
 */
public class StatisticGroupParser {

	/**
	 * Read the xml configuration file with the list of statistics to perform
	 * @param fileName 
	 * @return a list of statistics to execute
	 */
	public ArrayList<StatisticGroup> parse(String fileName) {
		
		ArrayList<StatisticGroup> groups = new ArrayList<StatisticGroup>();
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	        //String parameterName;
	        //Object parameterValue;
	        
	        try {
	            DocumentBuilder builder = factory.newDocumentBuilder();
	            Document doc = builder.parse(fileName);
	            Element root = doc.getDocumentElement();
	            NodeList rootNodeList = root.getChildNodes();
	            for (int i=0; i<rootNodeList.getLength();i++) {
	                if (rootNodeList.item(i).getNodeName().compareTo("group")==0){
	                	
	                	//create a statistic to hold the configuration read from the xml file
	                	StatisticGroup group = new StatisticGroup();
	                	String value;
	                	//retrieve all the parameters of the statistic
	                	NodeList statisticChild = rootNodeList.item(i).getChildNodes();
              	
	                	//iterate over all the child of the statistic node
	                	for(int j=0;j<statisticChild.getLength();j++){
	                		Node child = statisticChild.item(j);
	                		if(child.getNodeType() == Node.ELEMENT_NODE){
	                			try{
		                			if(child.getNodeName().compareTo("group-name")==0){
		                				value =child.getFirstChild().getNodeValue(); 
		                				group.setGroupName(value);
		                			}
		                			if(child.getNodeName().compareTo("group-title")==0){
		                				value =child.getFirstChild().getNodeValue(); 
		                				group.setGroupTitle(value);
		                			}
		                			if(child.getNodeName().compareTo("group-subtitle")==0){
		                				value =child.getFirstChild().getNodeValue(); 
		                				group.setGroupSubTitle(value);
		                			}
		                			if(child.getNodeName().compareTo("group-page")==0){
		                				value =child.getFirstChild().getNodeValue(); 
		                				group.setGroupPage(value);
		                			}
	                			}
	                			catch(NullPointerException nEx){
	                				System.out.println("Groups file bad formed: the tag " + child.getNodeName() + " is missing the value");
	                				System.exit(1);
	                			}
	                		}
	                	}
	                	groups.add(group);
	                }
	            }
	        }
	        catch (ParserConfigurationException pce) {
	            pce.printStackTrace(System.err);
	        }
	        catch (IOException ioe) {
	            ioe.printStackTrace(System.err);
	        }
	        catch (SAXException saxe) {
	            saxe.printStackTrace(System.err);
	        }
	        return groups;
	    }
	
	/**
	 * TODO
	 */
	public void writeStatistic(){
		
	}
	
	
}
