package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.ArrayList;


/**
 * The class is responsible of executing evary statistic defined into the statistics file or defined into the program
 * Start the creation of the html pages for the groups and the statistics
 * @author fangar
 *
 */
public class StatisticsGenerator {

	private HtmlStatisticPage html;
	private ArrayList<StatisticGroup> groups;
	private ArrayList<Statistic> stats;
	
	
	
	
	/**
	 * Execute all the statistic selected by the user over the schema, including the statistic defined into the program 
	 * @throws SQLException 
	 */
	public void executeStatistics() throws SQLException{
		
		RiskAnalysis risk;
		
		//parse the xml file with the statistic to execute
		StatisticGroupParser gparser = new StatisticGroupParser();
		this.groups = gparser.parse(StatisticsParameters.getGroups());
		html = new HtmlStatisticPage(groups);
		
		//parse the xml file with the statistic to execute
		StatisticFileParser fparser = new StatisticFileParser();
		this.stats = fparser.parse(StatisticsParameters.getStatList());
				
		for(Statistic stat : stats){
			stat.executeStatistic(stat);
			addStatToGroup(stat);
		}
		
		//list of statistics taking part in the risk analysis
		ArrayList<Statistic> statRiskAnalysis = new ArrayList<Statistic>();
		
		//execute the statistic defined into the statistics xml file ( statistic defined into the program and ready to be executed 
		for(String statistic : StatisticsParameters.getStats()){
			if(statistic.toLowerCase().equals("statrevisiontransition")){
				TablesAddedDeleted tables = new TablesAddedDeleted();
				tables.execute();
				addStatToGroup(tables);
			}
			if(statistic.toLowerCase().equals("stattableslifetime")){
				TablesLifetime life = new TablesLifetime();
				life.executeTablesLifetime();
				addStatToGroup(life);				
			}
			if(statistic.toLowerCase().equals("statcolumnslifetime")){
				ColumnsLifetime life = new ColumnsLifetime();
				life.executeColumnsLifetime();
				addStatToGroup(life);				
			}
			if(statistic.toLowerCase().equals("statmonthlyrevisions")){
				MonthlyRevisions monthly = new MonthlyRevisions();
				monthly.execute();
				addStatToGroup(monthly);
			}
			if(statistic.toLowerCase().equals("statrevisionslifetime")){
				RevisionsLifetime revision = new RevisionsLifetime();
				revision.executeRevisionLifetime();
				addStatToGroup(revision);
			}
			if(statistic.toLowerCase().equals("statprimarykeyschanges")){
				PrimaryKeyChanges primary = new PrimaryKeyChanges();
				primary.execute();
				addStatToGroup(primary);
				statRiskAnalysis.add(primary);
			}
			if(statistic.toLowerCase().equals("stattablerollbacks")){
				RolledBackTables roll = new RolledBackTables();
				roll.execute();
				addStatToGroup(roll);
				statRiskAnalysis.add(roll);
			}
			if(statistic.toLowerCase().equals("statindexchanges")){
				IndexChanges index = new IndexChanges();
				index.execute();
				addStatToGroup(index);
				statRiskAnalysis.add(index);
			}
			if(statistic.toLowerCase().equals("statforeignkeychanges")){
				ForeignKeyChanges fkey = new ForeignKeyChanges();
				fkey.execute();
				addStatToGroup(fkey);
				statRiskAnalysis.add(fkey);
			}
			if(statistic.toLowerCase().equals("statcolumnsevolution")){
				ColumnsAddedDeleted columns = new ColumnsAddedDeleted();
				columns.execute();
				statRiskAnalysis.add(columns);
			}
		}
		
		//perform the risk analysis
		//first of all set the available statistics taking part in the risk analysis
		//performs the other available statistics
		//retrieve the risk calculated
		risk = new RiskAnalysis();	
		risk.setStatsRisk(statRiskAnalysis);
		risk.executeRiskAnalysis();
		
		html.createStatIntroPage(risk);
		
		//create the page for each group of statistics
		for(StatisticGroup group: groups){
			html.createStatHtmlPage(group);
			System.out.println("Group " + group.getGroupName() + " html page created");
		}
	}
		
	
	/**
	 * Read the group which the statistic belongs and add the statistic to the group
	 * @param The statistic to add to the group
	 */
	private void addStatToGroup(Statistic stat){
		for(StatisticGroup group: groups){
			if(group.getGroupName().equals(stat.getStatGroup())){
				System.out.println("STAT ADDED TO GROUP:" + group.getGroupName());
				group.getStats().add(stat);
			}
		}
	}
	
	
	/**
	 * @param stat
	 */
	private Statistic getStatFromGroup(String name){
		
		for(StatisticGroup group: groups){
			for(Statistic stat : group.getStats()){
				if(stat.getStatName().equalsIgnoreCase(name)){
					return stat;
				}
			}
		}
		return null;
	}
	
	
	
	

}
		

