package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.Vector;

import org.jfree.data.category.DefaultCategoryDataset;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.Utils;


/**
 * Create and execute the statistic that performs an analysis over the database, retrieving how the databse is evolved between two 
 * following revision.
 * Retrieve the number of tables that have been added and deleted between a revision transition, in order to better understand the
 * databse evolutions
 * @author fangar
 *
 */
public class TablesAddedDeleted extends Statistic{

	/**
	 * Set the default filed for the statistic
	 */
	public TablesAddedDeleted() throws SQLException{
		
		//create the statistic
		//set the statistic parameters
		setTitle("Table Added and Deleted");
		setStatDescription("The statistic show the number of tables added and deleted every change of revision;\n The tables count statistic doesn't show " +
				"the detail of the transition between one revision and the other, while in this way it is possibile to better understand the change between " +
				"the revisions looking into the modifications occured to the database schema");
		setSqlStmt("The sql compare the name of the tables of every revision with the table name of the previuos and following revision");
		setRange("Added/Deleted");
		setDomain("Revisions");
		setStatName("statTablesAddedDeleted");
		setChartOrientation("v");
		setChartType("bar");
		setTitlePage("Statistic: Tables Added/Deleted");
		setStatPage(ApplicationParameters.getDbbasename() + getStatName() + ".html");
		setStatGroup("tables");
		
		
	}
	
	
	/**
	 * Execute the statistic
	 * @throws SQLException
	 */
	public void execute() throws SQLException{
		setDs(addedDeletedDataset());
		executeStatistic(this);
	}
	
	
	/**
	 * Create the dataset for the statistic
	 * @return the dataset
	 * @throws SQLException
	 */
	private DefaultCategoryDataset addedDeletedDataset() throws SQLException{
		boolean toAdd;
		int addedDeleted = 0; 
		DefaultCategoryDataset ds = new DefaultCategoryDataset();
		//create the dataset
		Vector<String> dbs = getDb().getDbInstalled();
    	
		//create the CSV file and add the header to the file
		createCSVFile(this);
		addValueToCsv(this, new String[]{"DomainValue","DomainValueRevisions","DomainValueAdded/Deleted","RangeValue"});
		
		
		for(int count = 0;count<dbs.size()-1;count++){
			toAdd  = false;
			
			//retrieve the name of the tables for the revision and revision +1
			Vector<String> tablesPrev = getDb().getTables(dbs.get(count));
			Vector<String> tablesNext = getDb().getTables(dbs.get(count+1));
			
			//calculate the number of operations over the tables
			int tablesAdded = Utils.getDifference(tablesNext,tablesPrev);
			int tablesDeleted = Utils.getDifference(tablesPrev,tablesNext);
			
			//add value to cvs /tables added and deleted) form each revision
    		addValueToCsv(this, new String[]{String.valueOf(addedDeleted),dbs.get(count),"added",String.valueOf(tablesAdded)});
    		addValueToCsv(this, new String[]{String.valueOf(addedDeleted),dbs.get(count),"deleted",String.valueOf(tablesDeleted)});			

    		//add the value to the chart (dataset) only if there have been operations (added or deleted different from zero)
    		if((tablesAdded!=0) ||(tablesDeleted!=0)) {
				addValueToDataset(ds, tablesAdded, "added", String.valueOf(addedDeleted));
				addValueToDataset(ds, tablesDeleted*-1, "deleted", String.valueOf(addedDeleted));
				toAdd = true;
			}
			if(toAdd){
				addedDeleted++;
			}
		}
		return ds;
	}
	
}
