package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.Vector;

import edu.ucla.cs.dbevolution.ApplicationParameters;

public class TablesLifetime extends StatisticLifetime{

	public TablesLifetime() {
	
		super();
		setStatName("statTablesLifetime");
		setStatDescription("The statistics show the lifetimes of each table: the life of a table is the number of revision where the table is present"+
			"it doesn't necessary mean the table was live for consecutive revisions, " +
			"this is just a qualitative chart to represent the number of revision where table was present");
		setSqlStmt("SELECT TABLE_NAME, COUNT(*) FROM " + ApplicationParameters.getHeaderEvolutionTables() + "TABLES WHERE TABLE_SCHEMA LIKE ? GROUP BY TABLE_NAME ORDER BY count(*) DESC");
		setTitlePage("Statistic: Tables Lifetime");
		setRange("Lifetime");
		setDomain("Tables");
		setChartType("bar");
		setTitle("Tables Lifetime");
		setImageAlt("tables lifetime");
		setChartOrientation("h");
		setStatGroup("tables");
		
	}
	
	/**
	 * Call the executeLifetime method of the superclass giving in input the sql statement to retrieve the lifetime of the tables
	 * and the list of parameters
	 * @throws SQLException 
	 */
	public void executeTablesLifetime() throws SQLException{
		
		//sql statement for the max lifetime
		String sql = "SELECT max(count) FROM (SELECT TABLE_NAME AS t_name, COUNT(*) AS count FROM " + ApplicationParameters.getHeaderEvolutionTables() + "TABLES WHERE TABLE_SCHEMA LIKE ? GROUP BY t_name) AS x";
		//params for max lifetime
		String[] params = new String[]{ApplicationParameters.getDbbasename()+"%"};
		
		//retrieve the dataset values
		Vector<StatisticValue> values = getDb().GetStatCount(ApplicationParameters.getDbbasename(), this.getSqlStmt());
		
		//call the super method
		executeLifetimeStatistic(values, sql, params);
	}
	
	
	
}
