/**
 * Copyright (C) 2007 Giorgio Orsi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
package it.polimi.elet.cautils.configurator;

/**
 * @author Giorgio Orsi (orsi@elet.polimi.it)
 * @version 0.1
 */
public final class CAConfigurator {
    
    private static CACmdLineParser cmdLineParser;
    private static CAConfigFileParser configFileParser;
    private static boolean existing;
    
    /**
     * sets the instance flag to true. 
     */
    private CAConfigurator() {
	existing = true;
    }
    
    /**
     * @return A new instance of the configurator.
     * @throws InstantiationException If an instance of the configurator already exists.
     */
    public static CAConfigurator getInstance() throws InstantiationException{
	if (!existing) {
	    return (new CAConfigurator());
	}
	throw new InstantiationException("Only one configurator at time can be used");	
    }
    
    /**
     * @return A new instance of the command line parser.
     */
    public static CACmdLineParser getCmdLineParserInstance() {
	try {
	    cmdLineParser = CACmdLineParser.getInstance();
	    return (cmdLineParser);
	} 
	catch (InstantiationException ie) {
	    return (cmdLineParser);
	}
    }
    
    /**
     * @return A new instance of the config file parser.
     */
    public static CAConfigFileParser getConfigFileParserInstance() {
	try {
	    configFileParser = CAConfigFileParser.getInstance();
	    return (configFileParser);
	} 
	catch (InstantiationException ie) {
	    return (configFileParser);
	}
    }
}
